@@ -1,4 +1,4 @@
-/* $Id: packets.h,v 1.24 2004/05/20 16:55:30 stpohle Exp $
+/* $Id: packets.h,v 1.31 2004/12/26 04:19:21 stpohle Exp $
  * network packets.. */
 
 #ifndef _PACKETS_H_
@@ -14,13 +14,16 @@
  * are between all clients so they will be forwarded.					*/
 enum _network_data {
     PKG_error = 0,
-	PKG_joingame,
-	PKG_playerid,
+	PKG_joingame,	// every packet below here will checked 
+					// if it comes from a orginal player
+	PKG_contest,	
+	PKG_playerid,	
     PKG_servermode,
     PKG_pingreq,
     PKG_pingack,
     PKG_getfield,
     PKG_getplayerdata,
+	PKG_teamdata,
     PKG_fieldline,
 	PKG_pkgack,
 	PKG_mapinfo,
@@ -86,6 +89,22 @@ struct pkg_ping {
 };
 
 
+struct pkg_contest {
+	struct pkgheader h;
+    signed char from;
+	signed char to;
+};
+
+
+struct pkg_teamdata {
+	struct pkgheader h;
+    signed char team_nr;
+	signed char col;
+	signed char wins;
+	char name[LEN_PLAYERNAME];
+};
+
+
 struct pkgdropitemelemt {
 	signed char x;
 	signed char y;
@@ -152,7 +171,7 @@ struct pkg_joingame {
 	signed char ver_major;		// Version
 	signed char ver_minor;		// Version
 	signed char ver_sub;		// Version
-	signed char netflags;
+	signed char secondplayer;	// this is the second player
 	char name[LEN_PLAYERNAME];
 	char password[LEN_PASSWORD];
 };
@@ -169,6 +188,9 @@ struct pkg_playerid {
 	signed char netflags;		// network flags
 	Sint16 points;
 	Sint16 wins;
+	signed char team_nr;		// team number
+	Sint16 team_points;			// team points
+	Sint16 team_wins;			// team wins
 };	
 
 
@@ -178,6 +200,7 @@ struct pkg_playerdata {
 	Sint16 points;				// points
 	Sint16 wins;				// how many times we win
 	signed char gfx_nr;			// the gfx number we want to use
+	signed char team_nr;		// teamnumber of the player
     _point pos;
     unsigned char bombs_n;
     unsigned char range;
@@ -185,6 +208,7 @@ struct pkg_playerdata {
     unsigned char d;
     unsigned char frame;
 	signed char dead_by;
+	signed char ready;			// if the player is ready for the game
 };
 
 
@@ -218,6 +242,7 @@ struct pkg_bombdata {
 struct pkg_quit {
 	struct pkgheader h;
 	signed char pl_nr;
+	signed char new_server;
 };
 
 
@@ -356,18 +381,20 @@ extern void do_mapinfo (struct pkg_mapinfo *map_pkg, _net_addr *addr);
 extern void do_tunneldata (struct pkg_tunneldata *tun_pkg, _net_addr *addr);
 extern void do_dropitems (struct pkg_dropitem *di_pkg, _net_addr *addr);
 extern void do_respawn (struct pkg_respawn *r_pkg, _net_addr *addr);
+extern void do_contest (struct pkg_contest *ct_pkg, _net_addr *addr);
+extern void do_teamdata (struct pkg_teamdata *td, _net_addr * addr);
 	
 extern void send_pkg (struct pkg *packet, _net_addr *addr);
-extern void send_playerid (_net_addr *addr, char *name, char *pladdr, char *plport, int p_nr, int gfx_nr, signed char netflags);
+extern void send_playerid (_net_addr *addr, char *name, char *pladdr, char *plport, int p_nr, int gfx_nr, int team_nr, signed char netflags);
 extern void send_servermode (_net_addr *addr, int pl_nr);
-extern void send_joingame (_net_addr * addr, char *name, int flags);
+extern void send_joingame (_net_addr * addr, char *name, int secondplayer);
 extern void send_error (_net_addr *addr, char *text);
 extern void send_field (_net_addr *addr, int x, int y, _field * field);
 extern void send_ping (_net_addr *addr, int data, unsigned char typ);
 extern void send_playerdata (_net_addr *addr, int p_nr, _player * pl);
 extern void send_playermove (_net_addr *addr, int p_nr, _player * pl);
 extern void send_bombdata (_net_addr *addr, int p, int b, _bomb * bomb);
-extern void send_quit (_net_addr *addr, int pl_nr);
+extern void send_quit (_net_addr *addr, int pl_nr, int new_server);
 extern void send_getfield (_net_addr *addr, int line);
 extern void send_fieldline (_net_addr *addr, int line);
 extern void send_getplayerdata (_net_addr *addr, int pl);
@@ -381,6 +408,8 @@ extern void send_updateinfo(_net_addr *addr);
 extern void send_tunneldata (_net_addr *addr, int tunnelnr, int x, int y);
 extern void send_dropitems (_net_addr *addr, int pl_nr, _flyingitem **fitems, int cnt);
 extern void send_respawn (_net_addr * addr, int plnr);
+extern void send_contest (_net_addr * addr, int from, int to, int ackreq);
+extern void send_teamdata (_net_addr * addr, int team_nr);
 
 extern void fwd_pkg (struct pkg *packet, _net_addr *addr);
 
