@@ -1,4 +1,4 @@
-/* $Id: bomb.c,v 1.57 2004/05/20 16:55:30 stpohle Exp $ */
+/* $Id: bomb.c,v 1.61 2004/12/01 02:41:54 stpohle Exp $ */
 /* everything what have to do with the bombs */
 
 #include "bomberclone.h"
@@ -25,7 +25,9 @@ draw_bomb (_bomb * bomb)
     }
 
     if (bomb->state != BS_trigger || ((bomb->state == BS_trigger) && (bomb->to < bman.bomb_tickingtime))) {
-        /* check the framenumber */
+        /*
+		 * check the framenumber
+		 */
         bomb->frame += (timefactor / 3.0);
         if (bomb->frame < 0 || bomb->frame >= gfx.bomb.frames)
             bomb->frame = 0.0f;
@@ -34,7 +36,9 @@ draw_bomb (_bomb * bomb)
     dest.w = src.w = gfx.bomb.image->w;
     dest.h = src.h = gfx.block.y;
     if (bomb->mode == BM_kicked) {
-		/* draw the kicked bomb */
+		/* 
+		 * draw the kicked bomb 
+		 */
         w = sqrt (bomb->speed);
         w *= absol (sin (w));
         x1 = bomb->dest.x - bomb->pos.x;
@@ -89,6 +93,13 @@ draw_bomb (_bomb * bomb)
 };
 
 
+
+/*
+ * the bomb is going to explode, prepare all values,
+ * set: ex_nr    - explosion number for the bomb, and all resulting explosions
+ *      to       - timeout
+ *      firer[d] - range of the bomb. 
+ */
 void
 bomb_explode (_bomb *bomb, int net)
 {
@@ -98,14 +109,22 @@ bomb_explode (_bomb *bomb, int net)
 
     if (bomb->ex_nr == -1)
         bomb->ex_nr = bman.last_ex_nr++; // set bomb explosion id
-
+	
     players[bomb->id.p].bomb_lastex = bomb->id.b;
-
     bomb->to = EXPLOSIONTIMEOUT; /* set the timeout for the fireexplosion */
     bomb->state = BS_exploding;
+
+	explosion_check_field ((int)bomb->pos.x, (int)bomb->pos.y, bomb);
+
     for (d = 0; d < 4; d++) {
-        bomb->firer[d] = 0;
-        bomb->firerst[d] = -1;
+		bomb->firer[d] = 0.0f;
+		bomb->firemaxr[d] = 0;
+		map.field[(int)bomb->pos.x][(int)bomb->pos.y].ex[d].count++;
+   		map.field[(int)bomb->pos.x][(int)bomb->pos.y].ex[d].frame = 0.0f;
+		map.field[(int)bomb->pos.x][(int)bomb->pos.y].ex_nr = bomb->ex_nr;
+		map.field[(int)bomb->pos.x][(int)bomb->pos.y].ex[d].bomb_p = bomb->id.p;
+		map.field[(int)bomb->pos.x][(int)bomb->pos.y].ex[d].bomb_b = bomb->id.b;
+		stonelist_add ((int)bomb->pos.x, (int)bomb->pos.y);
     }
 
     if (GT_MP_PTPM && net)      /* from now on only the server let the bomb explode */
@@ -223,12 +242,11 @@ bomb_move (_bomb * bomb)
 }
 
 
-int
+void
 bomb_loop ()
 {
     int p,
-      i,
-      b = 0;
+      i;
     _player *player;
     _bomb *bomb;
 
@@ -263,26 +281,24 @@ bomb_loop ()
                     if (bomb->mode != BM_normal)
                         bomb_action (bomb);
 
-                    b++;        // Count ticking Bombs for Return value
                     break;
 
                 case BS_exploding:
                     if (bomb->to > 0.0f) {
-                        do_explosion (bomb);
+                        explosion_do (bomb);
                     }
-                    if (bomb->to <= 0.0f) { // explosion done
-                        restore_explosion (bomb);
+                    else if (bomb->to <= 0.0f) { // explosion done
+                        explosion_restore (bomb);
                         bomb->to = 0.0f;
                         bomb->state = BS_off;
                     }
                     bomb->to -= timediff;
-                    b++;
                     break;
                 }
             }
         }
     }
-    return b;
+    return;
 };
 
 
@@ -314,50 +330,41 @@ get_bomb_on (float x, float y, _point bombs[])
 };
 
 
-
-void
-restore_explosion (_bomb * bomb)
-{
+/*
+ * restore the bombexplosion,
+ * will be excecuted after a explosion has finished.
+ * delete all old explosion data from the field.
+ */
+void explosion_restore (_bomb *bomb) {
     int i,
       d,
-      dx = 0,
-        dy = 0,
-        _x,
-        _y;
-
+      _x,
+      _y;
+	
+	d_printf ("explosion_restore: %f,%f\n", bomb->pos.x, bomb->pos.y);	
     for (d = 0; d < 4; d++) {
-        switch (d) {
-        case (left):
-            dx = -1;
-            dy = 0;
-            break;
-        case (right):
-            dx = 1;
-            dy = 0;
-            break;
-        case (up):
-            dx = 0;
-            dy = -1;
-            break;
-        case (down):
-            dx = 0;
-            dy = 1;
-            break;
-        }
-
         _x = bomb->pos.x;
         _y = bomb->pos.y;
+
+		if (map.field[_x][_y].ex[d].count > 0)
+			map.field[_x][_y].ex[d].count--;
+		map.field[_x][_y].ex[d].frame = 0.0f; // reset the framenumber
+		if (d==3) {
+			stonelist_add (_x, _y);
+			// printf ("restore: %d,%d\n", _x, _y);
+		}
 		
 		/* with every field where was an fire on it decrease the ex[].count value 
 		 * and force an drawing of this field */
-        for (i = 0; i < bomb->firer[d]; i++) {
-            if (--map.field[_x][_y].ex[d].count == 0) // there was only one explosion so 
-                map.field[_x][_y].ex[d].frame = 0; // reset the framenumber
+        for (i = 0; i < rintf(bomb->firer[d]); i++) {	// lower the number of explosions
+            _x = _x + dir_change[d].x;
+            _y = _y + dir_change[d].y;
 
+            if (map.field[_x][_y].ex[d].count > 0)
+				map.field[_x][_y].ex[d].count--;
+			map.field[_x][_y].ex[d].frame = 0.0f; // reset the framenumber
             stonelist_add (_x, _y);
-
-            _x = _x + dx;
-            _y = _y + dy;
+			// printf ("restore: %d,%d c=%d\n", _x, _y, map.field[_x][_y].ex[d].count);
         }
 
         /* delete the stone completly if there was any in the way 
@@ -370,11 +377,12 @@ restore_explosion (_bomb * bomb)
             if (map.field[_x][_y].special != FT_nothing) {
                 map.field[_x][_y].type = map.field[_x][_y].special;
                 map.field[_x][_y].special = FT_nothing;
-                d_printf ("field_explode (%d,%d) ex_nr = %d\n", _x, _y, map.field[_x][_y].ex_nr);
             }
             else
                 map.field[_x][_y].type = FT_nothing;
+			// d_printf ("field_explode (%d,%d) ex_nr = %d\n", _x, _y, map.field[_x][_y].ex_nr);
 
+			// printf ("380: d = %d, (%d,%d)\n", d, _x, _y);
             stonelist_add (_x, _y);
 
             if (GT_MP_PTPM)     /* send only if we are the master */
@@ -392,6 +400,95 @@ restore_explosion (_bomb * bomb)
 };
 
 
+
+/*
+ * draw the explosion as far as she got 
+ */
+void
+explosion_draw (_bomb * bomb)
+{
+    int d,
+      r;
+    _point p;
+
+    for (d = 0; d < 4; d++) {
+        p.x = bomb->pos.x;
+        p.y = bomb->pos.y;
+
+        for (r = 0; r < bomb->firer[d]; r++) {
+			map.field[p.x][p.y].ex[d].bomb_p = bomb->id.p;
+			map.field[p.x][p.y].ex[d].bomb_b = bomb->id.b;
+            map.field[p.x][p.y].ex[d].frame += timefactor;
+            if (map.field[p.x][p.y].ex[d].frame >= gfx.fire.frames)
+                map.field[p.x][p.y].ex[d].frame = 0.0f;
+            stonelist_add (p.x, p.y);
+            p.x += dir_change[d].x;
+            p.y += dir_change[d].y;
+        }
+    }
+}
+
+
+
+/*
+ * calculate the explosion itself,
+ * check the direction of the explosion and and and
+ *
+ * as long as the explosion grows, make sure the explosion will reach it's 'maximum' 
+ * (MAX_RANGE) end at 1.5 seconds. Make sure all explosion will have the same speed.
+ *    range += (MAX_RANGE/1.0) * timediff;
+ * 
+ */
+void explosion_do (_bomb *bomb) {
+	int d, dx, dy, ftype;
+	float range_grow = (timediff * (float) MAX_RANGE)/EXPLOSION_GROW_SPEED;
+	float new_range;
+	float step;
+	
+    if (bomb->state == BS_exploding) {
+		for (d = 0; d < 4; d++) {
+			if (bomb->firemaxr[d] == 0 && bomb->firer[d] < bomb->r) {
+				
+				new_range = bomb->firer[d] + range_grow;
+				if (new_range > bomb->r) new_range = bomb->r;
+
+				/* check all fields between [dx,dy] and [odx,ody] */
+				while (bomb->firemaxr[d] == 0 && (bomb->firer[d] < new_range)) {
+					step = new_range - bomb->firer[d];
+
+					if (step > 1.0f) {
+						step = 1.0f;
+					}
+					bomb->firer[d] += step;
+					// printf ("d:%d,   Cur_Range:%f  New_Range:%f\n", d, bomb->firer[d], new_range);
+					
+					dx = rintf(bomb->pos.x + dir_change[d].x * bomb->firer[d]);
+					dy = rintf(bomb->pos.y + dir_change[d].y * bomb->firer[d]);
+					
+					ftype = explosion_check_field (dx, dy, bomb);
+					if (ftype == FT_nothing || ftype == FT_tunnel) {
+						if (map.field[dx][dy].ex[d].count == 0 || map.field[dx][dy].ex_nr != bomb->ex_nr)
+							map.field[dx][dy].ex[d].count++;
+   		        		map.field[dx][dy].ex[d].frame = 0.0f;
+						map.field[dx][dy].ex_nr = bomb->ex_nr;
+						map.field[dx][dy].ex[d].bomb_p = bomb->id.p;
+						map.field[dx][dy].ex[d].bomb_b = bomb->id.b;
+						stonelist_add (dx, dy);
+					}
+					else {
+						bomb->firemaxr[d] = 1;
+						bomb->firer[d] = rintf (bomb->firer[d]);
+					}
+				}
+			}
+
+		}
+        explosion_draw (bomb);
+	}
+};
+
+
+
 /* check the field if there is another bomb stone or wathever
  * if a bomb is found let this one explode, on a player well this player
  * will die and if a stone was found, start with the stone explosion 
@@ -415,7 +512,7 @@ int explosion_check_field (int x, int y, _bomb *bomb)
         tmpbomb = &players[bo[i].x].bombs[bo[i].y];
         if (tmpbomb != bomb && tmpbomb->state != BS_exploding) {
             tmpbomb->ex_nr = bomb->ex_nr; // set the ex_nr to identify explosions
-            bomb_explode (&players[bo[i].x].bombs[bo[i].y], 1);
+            bomb_explode (tmpbomb, 1);
         }
     }
 
@@ -442,105 +539,6 @@ int explosion_check_field (int x, int y, _bomb *bomb)
 };
 
 
-/* draw the explosion as far as she got */
-void
-draw_explosion (_bomb * bomb)
-{
-    int d,
-      r,
-      dx,
-      dy;
-    _point p;
-
-    for (d = 0; d < 4; d++) {
-        switch (d) {
-        case (left):
-            dx = -1;
-            dy = 0;
-            break;
-        case (right):
-            dx = 1;
-            dy = 0;
-            break;
-        case (up):
-            dx = 0;
-            dy = -1;
-            break;
-        default:
-            dx = 0;
-            dy = 1;
-            break;
-        }
-        p.x = bomb->pos.x;
-        p.y = bomb->pos.y;
-
-        for (r = 0; r < bomb->firer[d]; r++) {
-			map.field[p.x][p.y].ex[d].bomb_p = bomb->id.p;
-			map.field[p.x][p.y].ex[d].bomb_b = bomb->id.b;
-            map.field[p.x][p.y].ex[d].frame += timefactor;
-            if (map.field[p.x][p.y].ex[d].frame >= gfx.fire.frames)
-                map.field[p.x][p.y].ex[d].frame = 0.0f;
-            stonelist_add (p.x, p.y);
-            p.x += dx;
-            p.y += dy;
-        }
-    }
-}
-
-/* do the bombexplosion itself, with every loop for one explosion 
- * add one more distance from the bomb if no stones are in the way */
-void
-do_explosion (_bomb *bomb)
-{
-    int dx = 0,
-        dy = 0,
-        d;
-
-    for (d = 0; d < 4; d++) {
-        switch (d) {
-        case (left):
-            dx = -1;
-            dy = 0;
-            break;
-        case (right):
-            dx = 1;
-            dy = 0;
-            break;
-        case (up):
-            dx = 0;
-            dy = -1;
-            break;
-        case (down):
-            dx = 0;
-            dy = 1;
-            break;
-        }
-
-        if (bomb->firer[d] <= bomb->r) {
-            int checkfield;
-
-            dx = bomb->firer[d] * dx;
-            dy = bomb->firer[d] * dy;
-
-            checkfield = explosion_check_field (bomb->pos.x + dx, bomb->pos.y + dy, bomb);
-            if ((checkfield == FT_nothing || checkfield == FT_tunnel)
-                && bomb->firerst[d] == -1) {
-                bomb->firer[d]++;
-                map.field[(int) bomb->pos.x + dx][(int) bomb->pos.y + dy].ex[d].count++;
-                map.field[(int) bomb->pos.x + dx][(int) bomb->pos.y + dy].ex[d].frame =
-                    bomb->firer[d];
-            }
-            else {
-                bomb->firerst[d] = bomb->firer[d];
-                stonelist_add (bomb->pos.x + dx, bomb->pos.y + dy);
-            }
-        }
-    }
-
-    /* with a slow pc disable this --- maybe option over a config menu */
-    if (bomb->state == BS_exploding)
-        draw_explosion (bomb);
-};
 
 
 inline void
