@@ -1,4 +1,4 @@
-/* $Id: configuration.c,v 1.62 2004/06/13 23:34:10 stpohle Exp $
+/* $Id: configuration.c,v 1.69 2004/12/26 04:19:20 stpohle Exp $
  * configuration */
 
 #include <SDL.h>
@@ -14,20 +14,60 @@
 #include "keyb.h"
 #include "player.h"
 
+
+/*
+ * try to find the datapath and set the variable bman.datapath
+ * test: 1) PACKAGE_DATA_DIR
+ *       2) ./data
+ *       3) ../data
+ */
+void config_get_datapath () {
+	FILE *f;
+	char filename[255];
+	
+    sprintf (bman.datapath, PACKAGE_DATA_DIR);
+    sprintf (filename, "%s/gfx/logo.png", bman.datapath);
+	f = fopen (filename, "r");
+	
+	if (!f) {
+	    sprintf (bman.datapath, "data");
+        sprintf (filename, "%s/gfx/logo.png", bman.datapath);
+        f = fopen (filename, "r");
+        if (!f) {
+	        sprintf (bman.datapath, "../data");
+    	    sprintf (filename, "%s/gfx/logo.png", bman.datapath);
+			f = fopen (filename, "r");
+			if (!f) {
+	            printf ("Can't find Datafiles.\n");
+    	        exit (1);
+       		}
+        }
+	}
+	fclose (f);
+}
+
+
+/*
+ * reset all variables and load all configs.
+ */
 void
 config_init (int argc, char **argv)
 {
 	SDL_Surface *icon_img;
 	char text[255], icon[255];
-    int i;
+    int i, j;
 
+	config_get_datapath ();
+	
     srand (((int) time (NULL))); // initialize randomgenerator
 
-    for (i = 0; i < MAX_PLAYERS; i++) {
-        players[i].gfx = NULL;  /* we will select them in the wait_for_players loop */
-        players[i].gfx_nr = -1; /* and even now in the singleplayer menu */
-    }
-
+	for (i = 0; i < MAX_TEAMS; i++) {
+		teams[i].col = i;
+		sprintf (teams[i].name, "Team %d", i+1);
+		for (j = 0; j < MAX_PLAYERS; j++)
+			teams[i].players[j] = NULL;
+	}
+	
     stonelist_del ();
 	chat.oldscreen = NULL;
 	chat.active = 0;
@@ -57,13 +97,13 @@ config_init (int argc, char **argv)
     gfx.res.x = 640;
     gfx.res.y = 480;
     gfx.bpp = 16;
+	gfx.players = NULL;
 	bman.password[0] = 0;
 	bman.passwordenabled = 0;
     map.tileset[0] = 0;
     map.random_tileset = 1;
     map.size.x = 25;
     map.size.y = 17;
-    sprintf (bman.datapath, PACKAGE_DATA_DIR);
     map.map[0] = 0;
     map.map_selection = 2;
     map.type = -1;
@@ -97,7 +137,7 @@ config_init (int argc, char **argv)
         ReadPrgArgs (argc, argv);
         gfx_init ();
         draw_logo ();
-        playermenu ();
+        playernamemenu ();
     }
     else {
         ReadPrgArgs (argc, argv);
@@ -245,9 +285,6 @@ config_read ()
             value[511] = 0;
             strcpy (map.map, value);
         }
-        if (!strcmp (keyword, "firewall")) {
-            bman.firewall = atoi (value);
-        }
         if (!strcmp (keyword, "udpport")) {
             if (strlen (value) > LEN_PORT) {
                 d_printf
@@ -333,6 +370,18 @@ config_read ()
             bman.dropitemsondeath = atoi (value);
         }
 		
+		for (i = 0; i < MAX_TEAMS; i++) {
+			char txt[255];
+			sprintf (txt,"teamcol%d", i);
+	        if (!strcmp (keyword, txt)) {
+    	        teams[i].col = atoi (value);
+        	}
+			sprintf (txt,"teamname%d", i);
+	        if (!strcmp (keyword, txt)) {
+    	        strncpy (teams[i].name,value, LEN_PLAYERNAME);
+        	}
+		}
+		
 		/*
 		 * keyboard config, i will give names to the keys insteed of the numbers,
 		 * this is done to add more keys to the game without destroying the config.
@@ -385,7 +434,9 @@ int
 config_write ()
 {
     FILE *config;
+	int i;
     char filename[512];
+	
 #ifdef _WIN32
     sprintf (filename, "%sbomberclone.cfg", s_gethomedir ());
 #else
@@ -398,7 +449,6 @@ config_write ()
     fprintf (config, "fullscreen=%d\n", gfx.fullscreen);
     fprintf (config, "tileset=%s\n", map.tileset);
     fprintf (config, "mapname=%s\n", map.map);
-    fprintf (config, "firewall=%d\n", bman.firewall);
     fprintf (config, "udpport=%s\n", bman.port);
     fprintf (config, "ai_players=%d\n", bman.ai_players);
     fprintf (config, "fieldsizex=%d\n", map.size.x);
@@ -429,7 +479,11 @@ config_write ()
 	fprintf (config, "start_range=%d\n", bman.start_range);
 	fprintf (config, "start_speed=%f\n", bman.start_speed);
 	fprintf (config, "bomb_ticking=%f\n", bman.bomb_tickingtime);
-	fprintf (config, "dropitemsondeath=%d\n", bman.dropitemsondeath);
+
+	for (i = 0; i < MAX_TEAMS; i++) {
+		fprintf (config, "teamcol%d=%d\n", i, teams[i].col);
+		fprintf (config, "teamname%d=%s\n", i, teams[i].name);
+	}
 
 	/*
 	 * keyboard config
@@ -548,7 +602,7 @@ config_menu ()
     while (menuselect != -1 && bman.state != GS_quit) {
         menu = menu_new ("Configuration", 400, 300);
         menu_create_label (menu, "General Option", -1, 50, 1, COLOR_brown);
-        menu_create_button (menu, "Playermenu", 25, 85, 150, 1);
+        menu_create_button (menu, "Playernames", 25, 85, 150, 1);
         menu_create_button (menu, "Keyboard", 225, 85, 150, 2);
         menu_create_button (menu, "Video Setup", -1, 120, 200, 3);
         menu_create_label (menu, "Sound", 25, 154, 0, COLOR_brown);
@@ -576,7 +630,7 @@ config_menu ()
                 menuselect = -1;
             break;
 		case (1):				// player screen 
-			playermenu ();
+			playernamemenu ();
 			break;
 		case (2):				// keyboard settings
 			keyb_config ();
@@ -610,9 +664,6 @@ ReadPrgArgs (int argc, char **argv)
 			printf ("                      (Def.: 11000)\n");
             printf (" -ogcport PORT      - set the local OGC Port (Def.: 11100)\n");
             printf (" -ogc 0/1           - Enable/Disable OGC\n");
-            printf (" -firewall 0/1      - Enable/Disable the firewall settings\n");
-            printf ("                      Only set this if you get some problems\n");
-            printf ("                      with network games.\n");
 			printf (" -host              - start a network game\n");
 			printf (" -join              - go into the join menu\n");
 			printf (" -connect ADDRESS   - connect to a server\n");
@@ -623,8 +674,6 @@ ReadPrgArgs (int argc, char **argv)
             strncpy (bman.port, argv[++i], LEN_PORT);
         if (!strcmp (argv[i], "-ogcport"))
             strncpy (bman.ogc_port, argv[++i], LEN_PORT);
-        if (!strcmp (argv[i], "-firewall"))
-            bman.firewall = atoi (argv[++i]);
         if (!strcmp (argv[i], "-name"))
             strncpy (bman.playername, argv[++i], LEN_PLAYERNAME);
         if (!strcmp (argv[i], "-name2"))
