@@ -1,4 +1,4 @@
-/* $Id: game.c,v 1.88 2004/06/16 00:41:29 stpohle Exp $ 
+/* $Id: game.c,v 1.106 2004/12/26 04:19:20 stpohle Exp $ 
   game.c - procedures for the game. */
 
 #include <string.h>
@@ -38,7 +38,6 @@ game_draw_info ()
 		chat_show (4, 4.5*16, gfx.res.x - 8, gfx.offset.y - 4.5*16);
 		chat_setactive (0, 0);
 	}
-
 	
     if (bman.updatestatusbar) {
         redraw_logo (0, 0, gfx.res.x, (4.5 * 16));
@@ -47,23 +46,21 @@ game_draw_info ()
         dest.w = gfx.res.x;
         gfx_blitupdaterectadd (&dest);
 
-        bman.players_nr = 0;
-
         /* In Multiplayer mode draw Player names and 
            count the players who are still alife. */
         for (x = 0, j = 0, i = 0; i < MAX_PLAYERS; i++)
             if ((players[i].state & PSFM_used) != 0) {
 
                 if (players[i].gfx_nr != -1 && PS_IS_used (players[i].state)) {
-                    src.x = 3 * players[i].gfx->smal_size.x;
+                    src.x = 0;
                     src.y = 0;
-                    src.w = dest.w = players[i].gfx->smal_size.x;
-                    src.h = dest.h = players[i].gfx->smal_size.y;
+                    src.w = dest.w = players[i].gfx->small_image->w;
+                    src.h = dest.h = players[i].gfx->small_image->h;
 
                     dest.x = x;
                     dest.y = j - 4;
 
-                    SDL_BlitSurface (players[i].gfx->smal_image, &src, gfx.screen, &dest);
+                    SDL_BlitSurface (players[i].gfx->small_image, &src, gfx.screen, &dest);
                 }
 
                 sprintf (scrtext, "%10s:%2d", players[i].name, players[i].points);
@@ -73,9 +70,11 @@ game_draw_info ()
                     else
                         col = 3;
                 }
-                else {          // players is alife
-					col = 0;
-                    bman.players_nr++;
+                else {          // player is alife
+					if (bman.gametype == GT_team)
+						col = teams[players[i].team_nr].col;
+					else
+						col = 0;
                 }
 
                 font_draw (x, j, scrtext, 0, col);
@@ -95,13 +94,13 @@ game_draw_info ()
         sprintf (text, "Speed: %1.1f", players[bman.p_nr].speed*10);
         font_draw (x, 32, text, 0, 0);
         if (players[bman.p_nr].special.type != 0) {
+			col = players[bman.p_nr].special.type + FT_sp_trigger - 1;
             dest.x = x - 32;
             dest.y = 16;
-            dest.w = gfx.smal_special[players[bman.p_nr].special.type - 1]->w;
-            dest.h = gfx.smal_special[players[bman.p_nr].special.type - 1]->h;
-
-            SDL_BlitSurface (gfx.smal_special[players[bman.p_nr].special.type - 1], NULL,
-                             gfx.screen, &dest);
+            dest.w = gfx.menu_field[col]->w;
+            dest.h = gfx.menu_field[col]->h;
+			
+            SDL_BlitSurface (gfx.menu_field[col], NULL, gfx.screen, &dest);
         }
 
         if (bman.state == GS_ready && GT_MP_PTPM)
@@ -163,8 +162,20 @@ void game_keys_loop () {
 		/* don't go into the game_keys if there is no menu displayed */
 
 		if (GT_MP_PTPM && bman.state == GS_ready && keyb_gamekeys.state[BCK_pause]  && !keyb_gamekeys.old[BCK_pause]) {
-			/* Server is starting the game */
-            bman.state = GS_running;
+			/* Server is starting the game
+			 * check in multiplayer if all players are ready for the game
+			 */
+			int i, ready = 1;
+			
+			for (i = 0; i < MAX_PLAYERS; i++)
+				if (NET_CANSEND (i) && !players[i].ready)
+					ready = 0;
+			
+			if (ready)
+				bman.state = GS_running;
+			else
+				d_printf ("game_keys_loop: not all players are ready\n");
+
             net_send_servermode ();
             bman.updatestatusbar = 1; // force an update
 		}
@@ -183,9 +194,9 @@ void game_keys_loop () {
             bman.updatestatusbar = 1; // force an update
 			}
 		*/
-	
+		
 		if (keyb_gamekeys.state[BCK_esc] && !keyb_gamekeys.old[BCK_esc]) {
-			if (chat.active && (bman.state == GS_ready || bman.state == GS_running)) {
+			if (chat.active && (bman.state == GS_ready || bman.state == GS_running) && IS_LPLAYER2) {
 				chat.active = 0;
 				d_printf ("Chatmode Disabled\n");
 			}
@@ -207,7 +218,7 @@ void
 game_loop ()
 {
     SDL_Event event;
-    int done = 0, i, eventstate;
+    int done = 0, eventstate;
 
 	gfx_blitupdaterectclear ();
 	
@@ -224,6 +235,18 @@ game_loop ()
     d_gamedetail ("GAME START");
     draw_players ();
 
+	
+	if (bman.p_nr >= 0 && bman.p_nr < MAX_PLAYERS) {
+		players[bman.p_nr].ready = 1;
+		if (GT_MP_PTPS)
+			send_playerdata (&players[bman.p_servnr].net.addr, bman.p_nr, &players[bman.p_nr]);
+	}
+	if (bman.p2_nr >= 0 && bman.p2_nr < MAX_PLAYERS) {
+		players[bman.p2_nr].ready = 1;
+		if (GT_MP_PTPS)
+			send_playerdata (&players[bman.p_servnr].net.addr, bman.p2_nr, &players[bman.p2_nr]);
+	}
+	
     while (!done && (bman.state == GS_running || bman.state == GS_ready)) {
         if ((eventstate = SDL_PollEvent (&event)) != 0)
             switch (event.type) {
@@ -239,7 +262,12 @@ game_loop ()
 
 		game_keys_loop ();
 
-		chat_loop (&event);
+		if (GT_MP)
+			chat_loop (&event);
+		
+		if ((!IS_LPLAYER2) && (!chat.active))
+			chat_setactive (1, 1);
+			
         restore_players_screen ();
 		
 		player_check (bman.p_nr);			
@@ -269,16 +297,8 @@ game_loop ()
 
         /* check if there is only one player left and the game is in multiplayer mode
            and if there the last dieing animation is done */
-		for (i = 0; (i < MAX_PLAYERS && (!PS_IS_used (players[i].state) 
-										|| PS_IS_aiplayer (players[i].state)
-										|| (PS_IS_used (players[i].state) && !PS_IS_alife (players[i].state)))); i++);
-		if (i == MAX_PLAYERS)  /* only AI Players alife, make sure the game is going to an end */
-			bman.players_nr = 1;
-        if (((bman.players_nr < 2
-             && (GT_MP || (GT_SP && bman.ai_players > 0)))
-			|| (GT_SP && bman.ai_players == 0 && bman.players_nr < 1))
-			&& bman.timeout >= 0.0 && (bman.gametype == GT_bomberman || map.state != MS_normal))
-            bman.timeout = 0.0f;
+		if (game_check_endgame () && bman.timeout >= 0.0f)
+			bman.timeout = 0.0f;
 		
         if ((GT_SP || GT_MP_PTPM) && bman.timeout < -GAME_OVERTIMEOUT) {
             d_printf ("GAME: Game Over\n");
@@ -310,27 +330,131 @@ game_loop ()
 };
 
 
+/*
+ * check if we only one player left or only ai players are left.
+ * check also if we there is only one team alife
+ */
+#define ENDGAME_CHECK_AGAIN 1.0f
+int game_check_endgame () {
+	int res = 0;
+	static float loop;
+	
+	loop -= timediff;
+	
+	if (loop > 0.0f && loop < ENDGAME_CHECK_AGAIN) return 0;
+	loop = ENDGAME_CHECK_AGAIN;
+	
+	if (bman.gametype == GT_team) {
+		/*
+		 * Team Mode Calculation
+		 */
+		int t_nr; 				// teamnumber
+		int p_nr;				// playernumber
+		int h_team = 0;			// how many human teams are alife
+		int ateam = 0;			// teams which are alife
+		int h_team_last = -1; 	// last human team which was alife
+		int team_last = -1;		// last teams which was alift
+		_player *p;
+		
+		for (t_nr = 0; t_nr < MAX_TEAMS; t_nr++) for (p_nr = 0; p_nr < MAX_PLAYERS; p_nr++)
+				if (teams[t_nr].players[p_nr] != NULL) {
+					p = teams[t_nr].players[p_nr];
+					if (PS_IS_used (p->state) && PS_IS_alife (p->state)) {
+						if (team_last != t_nr) {
+							team_last = t_nr;
+							ateam++;
+						}
+						if ((!PS_IS_aiplayer (p->state)) && h_team_last != t_nr) {
+							h_team++;
+							h_team_last = t_nr;
+						}
+					}
+				}
+		
+		if (h_team < 1 || ateam < 2)
+			res = 1;
+	}
+	else if ((bman.gametype == GT_bomberman) || (map.state != MS_normal && bman.gametype == GT_deathmatch)) {
+		int p_nr;			// playernumber
+		int h_alife = 0;	// human players who are alife
+		int alife = 0;		// ai players who are alife
+		_player *p;
+		
+		for (p = &players[0], p_nr = 0; p_nr < MAX_PLAYERS; p_nr++, p++) {
+			if (PS_IS_used (p->state) && PS_IS_alife (p->state)) {
+				alife++;
+				if (!PS_IS_aiplayer (p->state))
+					h_alife++;
+			}
+		}
+		
+		if ((h_alife < 1) || (alife < 2))
+			res = 1;
+	}
+	
+	return res;
+};
+#undef ENDGAME_CHECK_AGAIN
+
 /* check which player won and free all unnneded data */
 void
 game_end ()
 {
     int i;
-
+	int cnt_left = 0;
+	
     gfx_free_players ();
     tileset_free ();
 	snd_music_stop ();	
     snd_free ();
 
-    /* count the points */
+    /* count the wins for the player, and if only one player 
+	 * left count the points too */
+	cnt_left = 0;
     for (i = 0; i < MAX_PLAYERS; i++)
         if (PS_IS_used (players[i].state)) {
             if (PS_IS_alife (players[i].state)) {
                 bman.lastwinner = i;
+				cnt_left++;
                 players[i].wins++;
-                players[i].points += bman.players_nr_s;
             }
         }
+	if (cnt_left == 1)
+		players[bman.lastwinner].points += bman.players_nr_s;
+	else bman.lastwinner = -1;
 
+	/* check which team was alife */
+	if (bman.gametype == GT_team) {
+		int t_nr;
+		
+		bman.lastwinner = -1;
+		cnt_left = 0;
+		
+		for (t_nr = 0; t_nr < MAX_TEAMS; t_nr++) for (i = 0; i < MAX_PLAYERS; i++) {
+			if (teams[t_nr].players[i] != NULL) 
+				if (PS_IS_alife(teams[t_nr].players[i]->state)) {
+					if (bman.lastwinner != t_nr) {
+						teams[t_nr].wins++;
+						cnt_left++;
+						bman.lastwinner = t_nr;
+					}
+				}
+		}
+		
+		if (cnt_left == 1) {
+			/* set the points */
+			cnt_left = 0;
+			for (t_nr = 0; t_nr < MAX_TEAMS; t_nr++) {
+				for (i = 0; (i < MAX_PLAYERS && teams[t_nr].players[i] != NULL); i++);
+				if (i < MAX_PLAYERS && teams[t_nr].players[i] != NULL)
+					cnt_left++;
+			}
+			teams[bman.lastwinner].points += cnt_left;
+		}
+		else 
+			bman.lastwinner = -1;
+	}
+	
 	if (GT_SP)
 		game_showresult ();
 		
@@ -380,6 +504,7 @@ game_start ()
         players[p].pos.x = -1;
         players[p].pos.y = -1;
 		players[p].tunnelto = 0.0f;
+		players[p].ready = 0;
 
 		/* all types of illnes turn them off */
 		for (i = 0; i < PI_max; i++)
@@ -413,42 +538,233 @@ game_start ()
 };
 
 
-/* show result of the game */
+/*
+ * Show results of the game 
+ * show the diffrent screens one for players and one for teams
+ */
+/* Teamplay */
+#define SHOWRESULT_TEAMHEAD 24
+#define SHOWRESULT_TEAMPLAYER 16
+#define SHOWRESULT_TEAMPLAYERWIDTH 150
+void game_showresultteam (int pos_x, int pos_y, int pos_w, int pos_h) {
+	int i, t_nr, p_nr;		// counter for teams and players
+	struct {
+		_team *team;				// pointer to the team
+		_player *pl[MAX_PLAYERS];	// players in the team (sorted)
+		int cnt;
+	} tdata[MAX_TEAMS];	// hold some team informations (sorted)
+	int t_count = 0, p_maxcount = 0, p_sumcount = 0;
+	int sx, sy, p_y, p_x , dx, dy, col, x;
+	SDL_Rect dest, src;
+	char text[255];
+	
+	/* sort all teams */
+	for (t_nr = 0; t_nr < MAX_TEAMS; t_nr++) {
+		tdata[t_nr].team = NULL;
+		tdata[t_nr].cnt = 0;
+	}
+	
+	for (t_nr = 0; t_nr < MAX_TEAMS; t_nr++) {
+		for (p_nr = 0; (p_nr < MAX_PLAYERS && teams[t_nr].players[p_nr] == NULL); p_nr++);
+
+		if (p_nr < MAX_PLAYERS && teams[t_nr].players[p_nr] != NULL) {
+			tdata[t_count].team = &teams[t_nr];
+			i = t_count;
+			
+			while (i > 0 && (tdata[i-1].team->wins < teams[t_nr].wins 
+							|| (tdata[i-1].team->wins == teams[t_nr].wins 
+								&& tdata[i-1].team->points < teams[t_nr].points))) {
+				tdata[i].team = tdata[i-1].team;
+				i--;
+				tdata[i].team = &teams[t_nr];
+			}
+			t_count++;
+		}
+	}
+	
+	/* sort all players dependsing on the number of wins they have */
+	for (t_nr = 0; t_nr < t_count; t_nr++) for (p_nr = 0, tdata[t_nr].cnt = 0; p_nr < MAX_PLAYERS; p_nr++) {
+		if (t_nr < t_count) {
+			if (tdata[t_nr].team->players[p_nr] != NULL && PS_IS_used (tdata[t_nr].team->players[p_nr]->state)) {
+				tdata[t_nr].pl[tdata[t_nr].cnt] = tdata[t_nr].team->players[p_nr];
+				i = tdata[t_nr].cnt;
+			
+				while (i > 0 && (tdata[t_nr].pl[i-1]->wins < tdata[t_nr].team->players[p_nr]->wins
+								|| (tdata[t_nr].pl[i-1]->wins == tdata[t_nr].team->players[p_nr]->wins 
+									&& tdata[t_nr].pl[i-1]->points < tdata[t_nr].team->players[p_nr]->points))) {
+					tdata[t_nr].pl[i] = tdata[t_nr].pl[i-1];
+					i--;
+					tdata[t_nr].pl[i] = tdata[t_nr].team->players[p_nr];
+				}
+				tdata[t_nr].cnt++;
+			}
+		}
+	}
+	
+	/* check the max number of players in one team and number of all players */
+	for (t_nr = 0, p_maxcount = 0; t_nr < t_count; t_nr++) /* t_count + 1 */
+		if (p_maxcount < tdata[t_nr].cnt) p_maxcount = tdata[t_nr].cnt;
+
+	for (p_sumcount = 0, p_nr = 0; p_nr < MAX_PLAYERS; p_nr++)
+		if (PS_IS_used (players[p_nr].state)) p_sumcount++;
+		
+	/* calculate the best view */
+	p_x = dx = dy = 0;
+	do {
+		p_x++;
+		p_y = 0;  // calc. again for this setting
+		for (t_nr = 0; t_nr < t_count; t_nr++) {
+			p_y += ceil ((float)(((float) tdata[t_nr].cnt) / ((float)p_x)));
+		}
+		if (p_y == 0) p_y = 1;
+		dy = (pos_h - (SHOWRESULT_TEAMHEAD * t_count)) / p_y;
+	} while (dy < SHOWRESULT_TEAMPLAYER);
+	
+	if (dy > 2*SHOWRESULT_TEAMPLAYER) dy = 2*SHOWRESULT_TEAMPLAYER;
+
+	/* draw everything */
+	sy = (pos_h - (SHOWRESULT_TEAMHEAD * t_count + dy * p_y)) / 2;
+	for (t_nr = 0; t_nr < t_count; t_nr++) {
+		sprintf (text, "%s Victorys %d (%d)", tdata[t_nr].team->name, tdata[t_nr].team->wins, tdata[t_nr].team->points);
+		sx = (pos_w - strlen (text) * font[0].size.x) / 2;
+		font_gfxdrawbold (pos_x + sx, pos_y + sy+3, text, 0, COLOR_brown, 1, 1);
+		font_gfxdraw (pos_x + sx, pos_y + sy+3, text, 0, COLOR_yellow, 2);
+		sy += SHOWRESULT_TEAMHEAD;
+		dx = pos_w / p_x;
+		sx = (dx - SHOWRESULT_TEAMPLAYERWIDTH)/2;
+		
+		for (col = 0, p_nr = 0; p_nr < tdata[t_nr].cnt; p_nr++) {
+			if (col == 0 || col >= p_x) {
+				if (col >= p_x) sy += dy;
+				col = 0;
+				x = sx;
+			}
+		
+			if (tdata[t_nr].pl[p_nr]->gfx != NULL) {
+				dest.x = pos_x + x;
+				dest.y = pos_y + sy;
+				src.w = dest.w = tdata[t_nr].pl[p_nr]->gfx->small_image->w;
+				src.h = dest.h = tdata[t_nr].pl[p_nr]->gfx->small_image->h;
+				src.x = 0;
+				src.y = 0;
+				gfx_blit (tdata[t_nr].pl[p_nr]->gfx->small_image, &src, gfx.screen, &dest, 1);
+			}
+			sprintf (text, "%s(%d/%d)", tdata[t_nr].pl[p_nr]->name, tdata[t_nr].pl[p_nr]->wins, tdata[t_nr].pl[p_nr]->points);
+			font_gfxdraw (pos_x + x + GFX_SMALLPLAYERIMGSIZE_X*2, pos_y + sy + 2, text, 0, 0, 2);
+			x += dx;
+			col++;
+		}
+		sy += dy;
+	}
+}
+#undef SHOWRESULT_TEAMHEAD
+#undef SHOWRESULT_TEAMPLAYER
+#undef SHOWRESULT_TEAMPLAYERWIDTH
+
+
+/* Bomberman/Deathmatch Version Play */
+#define SHOWRESULT_WIDTH 150
+#define SHOWRESULT_HEIGHT 60
+void game_showresultnormal (int pos_x, int pos_y, int pos_w, int pos_h) {
+    char text[255];
+	int i, p, x, y, pl_cnt = 0, 
+		pl_x, pl_y, 				// player in a row/col
+		dx, dy,						// distance
+		sx, sy, px;					// start view and position
+	
+	SDL_Rect dest, src;
+	_player *pl[MAX_PLAYERS];
+	
+
+	/* Sort the playerlist */
+	for (p = 0, pl_cnt = 0; p < MAX_PLAYERS; p++) 
+		if (PS_IS_used (players[p].state)) {
+			pl[pl_cnt] = &players[p];
+			i = pl_cnt;
+			
+			while (i > 0 && (pl[i-1]->wins < players[p].wins 
+							|| (pl[i-1]->wins == players[p].wins 
+								&& pl[i-1]->points < players[p].points))) {
+				pl[i] = pl[i-1];
+				i--;
+				pl[i] = &players[p];
+			}
+			pl_cnt++;
+		}
+	
+	if (pl_cnt == 0) {
+		/* we still haven't joined the game */
+	}
+		
+	/* calc the best view and start point */
+	pl_x = 0;
+	do {
+		pl_x++;
+		pl_y = ceil ((float)(((float) pl_cnt) / ((float)pl_x)));
+		if (pl_y == 0) pl_y++;
+		dy = pos_h / pl_y;
+	} while (dy < SHOWRESULT_HEIGHT);
+	dx = pos_w / pl_x;
+	
+	x = sx = (dx - SHOWRESULT_WIDTH)/2;
+	y = sy = (dy - SHOWRESULT_HEIGHT)/2;
+	px = 0;
+
+	d_printf ("game_showresultnormal: pl_x:%d, pl_y:%d, dx:%d, dy:%d\n", pl_x, pl_y, dx, dy);
+	
+	/* draw the playerlist */
+	for (i = 1, p = 0; p < pl_cnt; p++) {
+		if (PS_IS_used (pl[p]->state)) {
+			if (PS_IS_alife (pl[p]->state)) {
+			    font_gfxdrawbold (pos_x + x + GFX_MENUPLAYERIMGSIZE_X + 8, pos_y + y - 10, pl[p]->name, 0, COLOR_brown, 1, 1);
+		    	font_gfxdraw (pos_x + x + GFX_MENUPLAYERIMGSIZE_X + 8,pos_y + y - 10, pl[p]->name, 0, COLOR_yellow, 1);
+			}
+			else 
+				font_gfxdraw (pos_x + x + GFX_MENUPLAYERIMGSIZE_X,pos_y + y - 10, pl[p]->name, 0, COLOR_gray, 1);
+
+			sprintf (text, "%3d (%3d)", pl[p]->wins, pl[p]->points);
+			font_gfxdraw (pos_x + x + GFX_MENUPLAYERIMGSIZE_X, pos_y + y + 6, text, 0, 0, 1);
+			
+			if (pl[p]->gfx != NULL) {
+				dest.x = pos_x + x;
+				dest.y = pos_y + y - 16;
+				src.w = dest.w = pl[p]->gfx->menu_image->w;
+				src.h = dest.h = pl[p]->gfx->menu_image->h;
+				src.x = 0;
+				src.y = 0;
+				gfx_blit (pl[p]->gfx->menu_image, &src, gfx.screen, &dest, 1);
+			}
+			/* setup the new position */
+			y += (dy / pl_x);
+			x += dx;
+			px++;
+			if (px >= pl_x) {
+				px = 0;
+				x = sx;
+			}
+		}
+	}
+}
+#undef SHOWRESULT_HEIGHT
+#undef SHOWRESULT_WIDTH
+
+
 void game_showresult () {
-	SDL_Rect dest,
-      src;
     char text[255];
     SDL_Event event;
     Uint8 *keys;
-	int done = 0, keypressed = 0, x, y, i, p;
+	int done = 0, keypressed = 0, x, y;
 	
-    menu_displaytext ("Loading..", "Please Wait");
-    dest.x = dest.y = 0;
-    dest.w = gfx.res.x;
-    dest.h = gfx.res.y;
-    gfx_load_players (40, 40);
-
+	gfx_blitdraw ();
+		
     draw_logo ();
+	
     strcpy (text, "Game Result");
     x = (gfx.res.x - (font[2].size.x * strlen (text)) - 64) / 2;
     y = 0;
     font_drawbold (x, y, text, 2, 6, 2);
     font_draw (x, y, text, 2, 5);
 	y += font[2].size.x;
-	
-	for (i = 0, p = 0; p < MAX_PLAYERS; p++)
-		if (PS_IS_alife (players[p].state))
-			i++;
-
-	if (i == 1) 
-		strcpy (text, "Game Over");
-	else
-		strcpy (text, "Draw Game");
-
-    x = (gfx.res.x - (font[2].size.x * strlen (text)) - 64) / 2;
-    font_drawbold (x, y, text, 2, 6, 2);
-    font_draw (x, y, text, 2, 5);
-	y += font[2].size.x + 8;
 
 	strcpy (text, "[CTRL],[RETURN] or [STRG] for another game");
     x = (gfx.res.x - (font[1].size.x * strlen (text)) - 64) / 2;
@@ -459,46 +775,16 @@ void game_showresult () {
     x = (gfx.res.x - (font[1].size.x * strlen (text)) - 64) / 2;
     font_drawbold (x, gfx.res.y - font[0].size.y - 2, text, 0, COLOR_brown, 1);
     font_draw (x, gfx.res.y - font[0].size.y - 2, text, 0, COLOR_yellow);
-	
-	for (i = 1, p = 0; p < MAX_PLAYERS; p++) {
-		if (PS_IS_used (players[p].state)) {
-			if (!i) {
-				i = 1;
-				x = (gfx.res.x / 2) + 16;
-			}
-			else if (i) {
-				i = 0;
-				y = y + 80;
-				x = 16;
-			}
-			
-			if (PS_IS_alife (players[p].state)) {
-			    font_drawbold (x + 80, y + 4, players[p].name, 0, COLOR_brown, 1);
-		    	font_draw (x + 80, y + 4, players[p].name, 0, COLOR_yellow);
-			}
-			else 
-				font_draw (x + 80, y + 4, players[p].name, 0, COLOR_gray);
 
-			sprintf (text, "%3d", players[p].wins);
-			font_draw (x + 80 + (LEN_PLAYERNAME+2) * font[0].size.x, y + 4, text, 0, 0);
-			sprintf (text, "%3d", players[p].points);
-			font_draw (x + 80 + (LEN_PLAYERNAME+5) * font[0].size.x, y + 4, text, 0, 0);
-			
-			dest.x = x;
-			dest.y = y - 32;
-			src.w = dest.w = players[p].gfx->ani.w;
-			src.h = dest.h = players[p].gfx->ani.h;
-			src.x = players[p].gfx->ani.w * down;
-			src.y = 0;
-			gfx_blit (players[p].gfx->ani.image, &src, gfx.screen, &dest, 1);
-		}
-	}
-	
+	if (bman.gametype == GT_team) 
+		game_showresultteam (10,50,gfx.res.x-20,gfx.res.y-100);
+	else
+		game_showresultnormal (10,50,gfx.res.x-20,gfx.res.y-100);
+
 	gfx_blitdraw ();
 	SDL_Flip (gfx.screen);
-	gfx_free_players ();
 
-    while (!done) {
+    while (!done && bman.state != GS_quit) {
         /* do the keyboard handling */
         if (SDL_PollEvent (&event) != 0)
             switch (event.type) {
@@ -589,8 +875,11 @@ void game_menu_loop (SDL_Event *event, int eventstate) {
 			bman.timeout = -GAME_OVERTIMEOUT;
 		}
 		
-		else if (menu->focus->id == 3) 							/* End Game */
-			bman.state = GS_startup;
+		else if (menu->focus->id == 3) { 							/* End Game */
+			/* send network update */
+			if (GT_MP)	net_game_send_delplayer (bman.p_nr);
+			bman.state = GS_quit;
+		}
 		
 		else { 													/* Quit Menu */
 			menu_delete (menu);
