@@ -1,4 +1,4 @@
-/* $Id: menu.c,v 1.45 2004/06/15 15:04:37 stpohle Exp $
+/* $Id: menu.c,v 1.49 2004/09/13 22:04:24 stpohle Exp $
  * Menuhandling */
 
 #include "basic.h"
@@ -44,6 +44,7 @@ menu_new (char *title, int x, int y)
     menu->oldscreen = gfx_copyscreen (&menu->oldscreenpos);
     menu->focus = NULL;
     menu->looprunning = 0;
+	menu->focusto = MENU_FOCUSVIS_BLINKTO;
 
 	menu->oldkey = 1;
 	
@@ -55,6 +56,8 @@ menu_new (char *title, int x, int y)
 void
 menu_delete (_menu * menu)
 {
+
+	gfx_blitdraw ();   // to make sure nothing is left in the blitbuffer
     gfx_restorescreen (menu->oldscreen, &menu->oldscreenpos);
     gfx_blitdraw ();
     SDL_FreeSurface (menu->oldscreen);
@@ -446,7 +449,7 @@ menu_event_loop (_menu * menu, SDL_Event * event, int eventstate)
         case (SDL_QUIT):
             bman.state = GS_quit;
             done = 1;
-            return 1;
+            return -1;
             break;
         case (SDL_KEYDOWN):    /* focus next element */
             if (menu->oldkey == 0 && event->key.keysym.sym == SDLK_TAB) {
@@ -585,15 +588,18 @@ menu_dir_select (char *title, char *path, signed char dirflags)
     _charlist flist[MAX_DIRENTRYS];
     int flcnt,
       menuselect;
-    _charlist *selfile = NULL;
+    _charlist *selfile = flist;
     _menu *menu;
+	_menuitem *dirmi;
 
     flcnt = menu_create_dirlist (path, dirflags, flist, MAX_DIRENTRYS);
     menu = menu_new (title, 300, 300);
-    menu_create_list (menu, "Dir", -1, 50, 200, 200, flist, &selfile, 1);
+    dirmi = menu_create_list (menu, "Dir", -1, 50, 200, 200, flist, &selfile, 1);
     menu_create_button (menu, "OK", -1, 270, 150, 0);
-
+	menu_focus_id (menu, 1);
+	
     menuselect = menu_loop (menu);
+
     menu_delete (menu);
 
     if (menuselect < 0 || selfile - &flist[0] < 0 || selfile - &flist[0] >= flcnt)
