@@ -71,7 +71,6 @@ network_options ()
 #ifndef _WIN32
 		menu_create_bool (menu, "IPV6", 260, 60, 100, &net_ai_typ, 4);
 #endif
-		menu_create_bool (menu, "Firewall", 260, 100, 100, &bman.firewall, 5);
 		menu_create_bool (menu, "Private Game", 20, 220, 150, &bman.passwordenabled, 6);
 		menu_create_entry (menu, "Password:", 190, 220, 210, &bman.password, LEN_PASSWORD, MENU_entrytext, 7);
 		
@@ -104,7 +103,7 @@ network_options ()
 void
 multiplayer_firstrun ()
 {
-    int i;
+    int i, j;
     /* 
        reset some gamedata
      */
@@ -123,8 +122,17 @@ multiplayer_firstrun ()
         players[i].net.pingreq = 20;
         players[i].net.pingack = 22;
         players[i].state = 0;
+		players[i].team_nr = -1;
     }
 
+	for (i = 0; i < MAX_TEAMS; i++) {
+		teams[i].wins = 0;
+		teams[i].points = 0;
+		
+		for (j= 0; j < MAX_PLAYERS; j++)
+			teams[i].players[j] = NULL;
+	}
+	
     bman.lastwinner = -1;
     bman.players_nr_s = 1;
 };
@@ -148,12 +156,16 @@ void host_multiplayer_game () {
 	/* prepare the server */
     strncpy (players[0].name, bman.playername, LEN_PLAYERNAME);
     bman.p_nr = bman.p_servnr = 0;
+	team_choose (&players[0]);
     players[0].state = PSF_used;
     if (bman.notifygamemaster)
         send_ogc_update ();   /* send the information that we started an server */
 	
 	multiplayer_game ();
-	net_send_servermode ();
+
+    if (bman.notifygamemaster)
+        send_ogc_update ();   /* send the information that we started an server */
+
     network_shutdown ();
 };
 
@@ -161,7 +173,7 @@ void host_multiplayer_game () {
 /* the loop for the multiplayer games */
 void multiplayer_game () {
 	
-    while (bman.state != GS_startup && bman.state != GS_quit) {
+    while (bman.state != GS_startup && bman.state != GS_quit && bman.sock != -1) {
         /* check players and in_pl lists */
         wait_for_players ();
 
@@ -204,7 +216,7 @@ void multiplayer_game () {
 void
 join_multiplayer_game ()
 {
-	int i, old_ai_family;
+	int old_ai_family;
 	
 	old_ai_family = bman.net_ai_family;
 	
@@ -240,14 +252,9 @@ join_multiplayer_game ()
         	          LEN_PORT, bman.net_ai_family, &players[0].net.addr.sAddr);
 	
 		/* and join the server */
-		if (bman.firewall)
-			i = NETF_firewall;
-		else
-			i = 0;
-		
 		bman.p_nr = -1;
-    	send_joingame (&players[0].net.addr, bman.playername, i);	
-	
+    	send_joingame (&players[0].net.addr, bman.playername, 0);
+
 		/* go into the normal multiplayer loop */
 		multiplayer_game ();
 	}
