@@ -1,4 +1,4 @@
-/* $Id: network.c,v 1.63 2004/05/20 16:55:30 stpohle Exp $ */
+/* $Id: network.c,v 1.72 2004/12/26 04:19:20 stpohle Exp $ */
 /*
 	network routines.
 */
@@ -146,19 +146,28 @@ void
 network_shutdown ()
 {
     int i;
-
+	int new_server = bman.p_servnr;
 	d_printf ("network_shutdown\n");
 
     if (GT_MP_PTPM) {
 		d_printf ("Server Quit Send information\n");
+		
+		/* find new server */
+		for (i = 0; i < MAX_PLAYERS; i++) {
+			if (PS_IS_used(players[i].state) && PS_IS_netplayer(players[i].state) && i != bman.p_servnr && players[i].net.flags == 0)
+				new_server = i;
+		}
+
+		d_printf ("netword_shutdown: new server set to: %d\n", new_server);
+		
         for (i = 0; i < MAX_PLAYERS; i++)
             if (i != bman.p_servnr && PS_IS_netplayer (players[i].state) && !PS_IS_aiplayer (players[i].state))
-                send_quit (&players[i].net.addr, bman.p_servnr);
+                send_quit (&players[i].net.addr, bman.p_servnr, new_server);
     }
     else if (players[bman.p_servnr].net.addr.host[0] != 0) {
-        send_quit (&players[bman.p_servnr].net.addr, bman.p_nr);
+        send_quit (&players[bman.p_servnr].net.addr, bman.p_nr, bman.p_servnr);
 		if (IS_LPLAYER2)
-			send_quit (&players[bman.p_servnr].net.addr, bman.p2_nr);
+			send_quit (&players[bman.p_servnr].net.addr, bman.p2_nr, bman.p_servnr);
 	}
 
 	if (bman.notifygamemaster) {
@@ -307,15 +316,15 @@ draw_netupdatestate (char st)
 				redraw_logo (0, y, gfx.res.x, y + 50);
 
             	if (players[i].gfx_nr != -1) {
-                	dest.w = src.w = players[i].gfx->smal_size.x;
-                	dest.h = src.h = players[i].gfx->smal_size.y;
-                	src.x = players[i].gfx->smal_size.x * down;
+                	dest.w = src.w = players[i].gfx->small_image->w;
+                	dest.h = src.h = players[i].gfx->small_image->h;
+                	src.x = players[i].gfx->small_image->w * down;
                 	src.y = 0;
 
                 	dest.x = 50;
                 	dest.y = y;
 
-                	SDL_BlitSurface (players[i].gfx->smal_image, &src, gfx.screen, &dest);
+                	SDL_BlitSurface (players[i].gfx->small_image, &src, gfx.screen, &dest);
                 	gfx_blitupdaterectadd (&dest);
 				}
 
@@ -407,34 +416,37 @@ draw_netupdatestate (char st)
 }
 
 /*
-	used to update settings at startup 
+	used to update settings at startup
 */
 void
-net_change_playerid (int pl_nr, unsigned char senddata)
+net_send_playerid (int pl_nr)
 {
     int i;
+
+	d_printf ("net_send_playerid pl_nr:%d\n", pl_nr);
 	
     if (GT_MP_PTPM) {
         /*
            Send to all connected clients the update
          */
         for (i = 1; i < MAX_PLAYERS; i++)
-            if (!PS_IS_aiplayer (players[i].state) && players[i].net.addr.host[0] != 0)
+            if (NET_CANSEND (i))
                 send_playerid (&players[i].net.addr, players[pl_nr].name,
                                players[pl_nr].net.addr.host, players[pl_nr].net.addr.port,
-                               pl_nr, players[pl_nr].gfx_nr, players[pl_nr].net.flags);
+                               pl_nr, players[pl_nr].gfx_nr, players[pl_nr].team_nr, players[pl_nr].net.flags);
     }
     else {
         /*
-           Send only to the Server the update and only if in_nr == bman.in_nr
+           Send only to the Server the update and only if pn_nr == bman.p_nr
          */
-        if ((pl_nr == bman.p_nr || (IS_LPLAYER2 && pl_nr == bman.p2_nr)) && senddata)
+        if (pl_nr == bman.p_nr || (IS_LPLAYER2 && pl_nr == bman.p2_nr))
             send_playerid (&players[bman.p_servnr].net.addr, players[pl_nr].name,
                            players[pl_nr].net.addr.host, players[pl_nr].net.addr.port,
-                           pl_nr, players[pl_nr].gfx_nr, players[pl_nr].net.flags);
+                           pl_nr, players[pl_nr].gfx_nr, players[pl_nr].team_nr, players[pl_nr].net.flags);
     }
 
-    player_set_gfx (&players[pl_nr], players[pl_nr].gfx_nr);
+	if ((players[pl_nr].gfx_nr >= 0 && players[pl_nr].gfx != &gfx.players[players[pl_nr].gfx_nr]) || (players[pl_nr].gfx_nr == -1 && players[pl_nr].gfx != NULL))
+		player_set_gfx (&players[pl_nr], players[pl_nr].gfx_nr);
 };
 
 
@@ -578,6 +590,7 @@ net_transmit_gamedata ()
                     send_playerstatus (&players[bman.p_servnr].net.addr, bman.p_nr, 0, 0);
                 }
             }
+			
             if (net_istep == 0 && players[bman.p_nr].net.net_status == -1
                 && timestamp - downtimestamp > DOWNLOAD_TIMEOUT) {
                 /* server did not send informations back */
@@ -623,6 +636,10 @@ net_transmit_gamedata ()
 };
 
 
+/*
+ * send informations about a player too all connected players
+ * last_change: the player will not be send if (send_to_p_nr == bman.p_nr)
+ */
 void
 net_game_send_player (int p_nr)
 {
@@ -630,7 +647,7 @@ net_game_send_player (int p_nr)
 
     if (GT_MP_PTPM) {
         for (p = 0; p < MAX_PLAYERS; p++)
-            if (PS_IS_netplayer (players[p].state) && p != bman.p_nr && p != bman.p2_nr)
+            if (PS_IS_netplayer (players[p].state) && p != bman.p_nr && p != bman.p2_nr && p != p_nr)
                 send_playerdata (&players[p].net.addr, p_nr, &players[p_nr]);
     }
     else if (p_nr == bman.p_nr || p_nr == bman.p2_nr) {
@@ -703,22 +720,40 @@ net_game_send_field (int x, int y)
 void
 net_game_send_delplayer (int pl_nr) {
     int i;
+	int new_server = bman.p_servnr;
 
     d_printf ("net_game_send_delplayer (%d)\n", pl_nr);
 
     if (GT_MP_PTPM && (GS_WAITRUNNING || bman.state == GS_update)) {
+
+		/* find new server, if needed */
+		if (pl_nr == bman.p_servnr) {
+			for (i = 0; i < MAX_PLAYERS; i++) {
+				if (PS_IS_used(players[i].state) && PS_IS_netplayer(players[i].state) && i != bman.p_servnr && players[i].net.flags == 0)
+					new_server = i;
+			}
+			d_printf ("new_game_send_delplayer: new server set to: %d\n", new_server);
+		}
+		
 	    for (i = 0; i < MAX_PLAYERS; i++)
-    		if (NET_CANSEND(i) || pl_nr == i)
-                send_quit (&players[i].net.addr, pl_nr);
-		if (bman.notifygamemaster)
-			send_ogc_update ();
+   			if (NET_CANSEND(i) && i != bman.p_nr)
+       	        send_quit (&players[i].net.addr, pl_nr, new_server);
 		bman.updatestatusbar=1;
 	}
+	/* we have to send that one if our own players quit */
+	else if (pl_nr == bman.p_nr || pl_nr == bman.p2_nr) {
+		send_quit (&players[bman.p_servnr].net.addr, pl_nr, -1);
+	}
 
-    if (GT_MP_PTPM && bman.notifygamemaster)
-        send_ogc_update ();
-	
 	inpkg_delplayer (pl_nr);
+
+    if (GT_MP_PTPM) {
+		if (bman.notifygamemaster)
+        	send_ogc_update ();
+	
+		if (new_server >= 0 && new_server < MAX_PLAYERS)
+			bman.p_servnr = new_server;
+	}
 };
 
 
@@ -762,12 +797,28 @@ net_send_players ()
             for (i = 0; i < MAX_PLAYERS; i++)
                 send_playerid (&players[j].net.addr, players[i].name,
                                players[i].net.addr.host, players[i].net.addr.port, i,
-                               players[i].gfx_nr, players[i].net.flags);
+                               players[i].gfx_nr, players[i].team_nr, players[i].net.flags);
 
 };
 
 
 
+/* sends to everyone the teamdata */
+void
+net_send_teamdata (int team_nr)
+{
+    int j;
+
+	if (GT_MP_PTPS)
+		return;
+	
+	for (j = 0; j < MAX_PLAYERS; j++)
+		if (NET_CANSEND (j))
+			send_teamdata (&players[j].net.addr, team_nr);
+};
+
+
+
 void
 net_send_chat (char *text, signed char notigamesrv)
 {
@@ -915,7 +966,6 @@ void send_ogc_update () {
 			sprintf (status, "error");
 			break;
 	}
-	
 	ogc_sendgamestatus (bman.sock, "BomberClone", VERSION, bman.gamename, j, bman.maxplayer, status);
 };
 
