@@ -65,6 +65,8 @@ do_error (struct pkg_error *data, _net_addr * addr)
 {
     d_printf ("Network Error from %s:%s : '%s'\n", addr->host, addr->port, data->text);
 	
+	network_shutdown ();
+	
     menu_displaymessage ("Network Error", "Got Error from: %s:%s\nMessage:%s", addr->host, addr->port, data->text);
     if (data->nr == 1)
         return -1;
@@ -127,7 +129,8 @@ do_joingame (struct pkg_joingame *p_jg, _net_addr * addr)
 			if (!PS_IS_used (players[j].state) && freeslot == -1) freeslot = j;
 			if (strncmp (players[j].net.addr.host, addr->host, LEN_SERVERNAME) == 0
 				&& strncmp (players[j].net.addr.port, addr->port, LEN_PORT) == 0
-				&& p_jg->netflags == players[j].net.flags) i = j;
+				&& ((p_jg->secondplayer && (players[j].net.flags & NETF_local2) != 0)
+					|| (!p_jg->secondplayer && (players[j].net.flags & NETF_local2) == 0)))	i = j;
 		}
 		
 		if (i == -1) i = freeslot;
@@ -143,9 +146,14 @@ do_joingame (struct pkg_joingame *p_jg, _net_addr * addr)
                 d_printf ("     Player Added : Nr:[%d]   Name:%10s\n", i, p_jg->name);
                 pl->points = 0;
                 pl->wins = 0;
+				pl->team_nr = -1;
+				
+				team_choose (pl);
             }
             strncpy (pl->name, p_jg->name, LEN_PLAYERNAME);
-            player_set_gfx (pl, -1);
+            pl->gfx_nr = -1;
+			pl->gfx = NULL;
+			pl->state &= (0xFF - (PSF_alife + PSF_playing));
 
             /* Reset the network data */
             pl->net.timestamp = timestamp;
@@ -158,22 +166,27 @@ do_joingame (struct pkg_joingame *p_jg, _net_addr * addr)
             strncpy (pl->net.addr.port, addr->port, LEN_PORT);
             dns_filladdr (pl->net.addr.host, LEN_SERVERNAME, pl->net.addr.port,
                           LEN_PORT, bman.net_ai_family, &pl->net.addr.sAddr);
-            pl->net.flags = p_jg->netflags;
+			if (p_jg->secondplayer)
+				pl->net.flags = NETF_local2 + NETF_firewall;
+			else 
+				pl->net.flags = NETF_firewall;
             pl->net.addr.pl_nr = i;
             bman.players_nr_s++;
 
             /* send to the new client the servermode and the complete playerlist */
-            send_mapinfo (addr);
+			if ((!p_jg->secondplayer) && !(players[j].net.flags & NETF_local2))
+            	send_mapinfo (addr);
             send_servermode (addr, i); // with this packet the client know it'S pl_nr
 
             addr->pl_nr = i;
 
-            /* Send all connected players the new PlayerID */
+            /* Send all connected players the new PlayerID, except to the new player 
 			for (i = 0; i < MAX_PLAYERS; i++)
-				if (NET_CANSEND(i))
+				if (NET_CANSEND(i) && addr->pl_nr != i)
                     send_playerid (&players[i].net.addr, pl->name, pl->net.addr.host,
                                    pl->net.addr.port, pl->net.addr.pl_nr, pl->gfx_nr,
-                                   pl->net.flags);
+                                   pl->team_nr, pl->net.flags);
+			*/
         }
 
         else if (GS_WAITRUNNING) {
@@ -193,7 +206,7 @@ do_joingame (struct pkg_joingame *p_jg, _net_addr * addr)
 
 
 void
-send_joingame (_net_addr * addr, char *name, int flags)
+send_joingame (_net_addr * addr, char *name, int secondplayer)
 {
     struct pkg_joingame p_jg;
     int vmi,
@@ -210,17 +223,73 @@ send_joingame (_net_addr * addr, char *name, int flags)
     p_jg.ver_sub = vsu;
     p_jg.ver_major = vma;
     p_jg.ver_minor = vmi;
-	p_jg.netflags = flags;
+	p_jg.secondplayer = secondplayer;
     strncpy (p_jg.name, name, LEN_PLAYERNAME);
     strncpy (p_jg.password, bman.password, LEN_PASSWORD);
 
+	bman.firewall = 1;
     send_pkg ((struct pkg *) &p_jg, addr);
 };
 
 
+
 /***
- *** Packettype: pingack/pingreq
- *** Update Playerinformation, add players into the current game
+ *** Packettype: contest
+ *** Test the connection from a new player for the firewall flag.
+ *** This packet will be send from one player to the other the
+ *** current server will only get this packet if a connection test was positive
+ ***/
+void do_contest (struct pkg_contest *ct_pkg, _net_addr *addr) {
+	d_printf ("do_contest (pl_nr = %d) from=%d to=%d\n", addr->pl_nr, ct_pkg->from, ct_pkg->to);
+	
+	if (addr->pl_nr >= MAX_PLAYERS 
+		|| (addr->pl_nr == -1 && PS_IS_netplayer(players[ct_pkg->from].state))) {
+		d_printf ("     addr->pl_nr out of range (0-MAX_PLAYERS)\n");
+		return;
+	}
+
+	/* master will have to change the firewall flag on a player */	
+	if (GT_MP_PTPM) {
+		if (ct_pkg->from < 0 || ct_pkg->from >= MAX_PLAYERS) {
+				if ((ct_pkg->to < 0 || ct_pkg->to >= MAX_PLAYERS) && addr->pl_nr != ct_pkg->from)
+					d_printf ("     from or to value out of range (0-MAX_PLAYERS)\n");
+			return;
+		}
+		
+		players[ct_pkg->to].net.flags &= (0xFF - NETF_firewall);
+		net_game_send_player (ct_pkg->to);
+	}
+
+	/* if a client get this packet we send a packet
+	 * to the server that we have got this packet. */
+	else {
+		send_contest (&players[bman.p_servnr].net.addr, ct_pkg->from, bman.p_nr, 1);
+		bman.firewall = 0;
+	}
+}
+
+
+void send_contest (_net_addr * addr, int from, int to, int ackreq) {
+	struct pkg_contest ct_pkg;
+	
+	d_printf ("send_contest addr->id:%d, from:%d, to:%d\n", addr->pl_nr, from, to);
+
+    ct_pkg.h.typ = PKG_contest;
+	if (ackreq) ct_pkg.h.flags = PKGF_ackreq;
+	else ct_pkg.h.flags = 0;
+    ct_pkg.h.len = HTON16 (sizeof (struct pkg_contest));
+	
+	ct_pkg.from = from;
+	ct_pkg.to = to;
+	
+	send_pkg ((struct pkg *) &ct_pkg, addr);
+}
+
+
+/***
+ *** Packettype: playerid
+ *** Update Playerinformation, Teampoints of a Player, Points
+ ***
  ***
  *** if  : the pl_nr == -1 send the whole list to the sender
  *** else: set the new values for this player
@@ -238,29 +307,30 @@ do_playerid (struct pkg_playerid *p_id, _net_addr * addr)
 	/*
 	 * check if we have to send the whole playerlist to a client 
 	 */
-	if (GT_MP_PTPM && p_id->pl_nr == -1) {
+	if (GT_MP_PTPM && p_id->pl_nr == -1 && addr->pl_nr >= 0 && addr->pl_nr < MAX_PLAYERS) {
+         /* Send all connected players the new PlayerID, except to the new player */
+		pl = &players[addr->pl_nr];
 		for (i = 0; i < MAX_PLAYERS; i++)
-			send_playerid (addr, players[i].name, players[i].net.addr.host, players[i].net.addr.port, i, players[i].gfx_nr, players[i].net.flags);
+			if (NET_CANSEND(i) && addr->pl_nr != i)
+                send_playerid (&players[i].net.addr, pl->name, pl->net.addr.host,
+                               pl->net.addr.port, pl->net.addr.pl_nr, pl->gfx_nr,
+                               pl->team_nr, pl->net.flags);
+		
+		for (i = 0; i < MAX_PLAYERS; i++)
+			send_playerid (addr, players[i].name, players[i].net.addr.host, players[i].net.addr.port, i, players[i].gfx_nr, players[i].team_nr, players[i].net.flags);
 	}
+
 	/*
 	 * check if we have to update someones data
 	 */
 	else if ((GT_MP_PTPM && p_id->pl_nr >= 0 && p_id->pl_nr < MAX_PLAYERS && p_id->pl_nr != bman.p_nr && p_id->pl_nr != bman.p2_nr)
 			 || (GT_MP_PTPS && p_id->pl_nr >= 0 && p_id->pl_nr < MAX_PLAYERS)) {
-				 
+
         pl = &players[p_id->pl_nr];
-
-        pl->state = p_id->state;
-        if (p_id->pl_nr != bman.p_nr && PS_IS_used (pl->state))
-            pl->state |= PSF_net;
-        else
-            pl->state &= (0xff - PSF_net);
-
-        player_set_gfx (pl, p_id->gfx_nr);
-        strncpy (pl->name, p_id->name, LEN_PLAYERNAME);
+		
         pl->net.timestamp = timestamp;
         pl->net.pingreq = pl->net.pingack + 5;
-		pl->net.flags = p_id->netflags;
+		if (GT_MP_PTPS) pl->net.flags = p_id->netflags;
         if (p_id->host[0] != 0) {
             pl->net.addr.pl_nr = addr->pl_nr;
             strncpy (pl->net.addr.host, p_id->host, LEN_SERVERNAME);
@@ -275,34 +345,57 @@ do_playerid (struct pkg_playerid *p_id, _net_addr * addr)
 			strncpy (pl->net.addr.port, players[bman.p_servnr].net.addr.port, LEN_PORT);
 			memcpy (&pl->net.addr.sAddr, &players[bman.p_servnr].net.addr.sAddr, sizeof (struct _sockaddr));
 		}
+		
+		/* Check if we have to make a network test.. only client to client 
+		 * we won't check 2 players too because i't won't be possible to send
+		 * something to them. Start the timer for the connection test. */
+		if (GT_MP_PTPS && !(PS_IS_netplayer (pl->state)) && (PS_IS_netplayer (p_id->state))
+			&& p_id->pl_nr != bman.p_servnr && p_id->pl_nr != bman.p_nr
+			&& !(pl->net.flags & NETF_local2) && p_id->pl_nr != bman.p2_nr) {
+
+			send_contest (&pl->net.addr, bman.p_nr, -1, 0); // send contest without ackreq.
+				/* make sure we still get messages from the server, this is a 
+				 * work around for some hardware routers */
+			send_contest (&players[bman.p_servnr].net.addr, bman.p_nr, -1, 1); 
+		}
+        pl->state = p_id->state;
+        if (p_id->pl_nr != bman.p_nr && PS_IS_used (pl->state))
+            pl->state |= PSF_net;
+        else
+            pl->state &= (0xff - PSF_net);
+
+        if (pl->gfx_nr != p_id->gfx_nr) player_set_gfx (pl, p_id->gfx_nr);
+        strncpy (pl->name, p_id->name, LEN_PLAYERNAME);
 			
         if (GT_MP_PTPS) {
             pl->points = NTOH16 (p_id->points);
             pl->wins = NTOH16 (p_id->wins);
+			pl->team_nr = p_id->team_nr;
+			if (pl->team_nr >= 0 && pl->team_nr < MAX_TEAMS) {
+				teams[pl->team_nr].points = NTOH16(p_id->team_points);
+				teams[pl->team_nr].wins = NTOH16(p_id->team_wins);
+			}
         }
 
         /* Send all connected players the new PlayerID */
         if (GT_MP_PTPM && GS_WAITRUNNING && addr->pl_nr >= 0 && addr->pl_nr < MAX_PLAYERS)
-            net_change_playerid (addr->pl_nr, (GT_MP_PTPM));
+            net_send_playerid (addr->pl_nr);
 	}
 	
-    d_playerdetail ("*** PLAYER List ***");
+  //  d_playerdetail ("*** PLAYER List ***");
+	team_update ();
     bman.updatestatusbar = 1;
 };
 
 
 void
 send_playerid (_net_addr * addr, char *name, char *pladdr, char *plport,
-               int pl_nr, int gfx_nr, signed char netflags)
+               int pl_nr, int gfx_nr, int team_nr,  signed char netflags)
 {
     struct pkg_playerid p_id;
-    int vmi,
-      vma,
-      vsu;
 
     d_printf ("send_playerid SendTo: %s:%s (Name:%16s p_nr:%d)\n", addr->host,
               addr->port, name, pl_nr);
-    sscanf (VERSION, "%d.%d.%d", &vma, &vmi, &vsu);
 
     p_id.h.typ = PKG_playerid;
     p_id.h.flags = PKGF_ackreq;
@@ -330,17 +423,85 @@ send_playerid (_net_addr * addr, char *name, char *pladdr, char *plport,
         p_id.points = HTON16 (players[pl_nr].points);
         p_id.wins = HTON16 (players[pl_nr].wins);
         p_id.state = players[pl_nr].state;
+		p_id.team_nr = team_nr;
+		if (team_nr >= 0 && team_nr < MAX_TEAMS) {
+			p_id.team_points = teams[team_nr].points;
+			p_id.team_wins = teams[team_nr].wins;
+		}
     }
     else {
         p_id.points = 0;
         p_id.wins = 0;
         p_id.state = 0;
+		p_id.team_nr = -1;
+		p_id.team_points = 0;
+		p_id.team_wins = 0;
     }
 
     send_pkg ((struct pkg *) &p_id, addr);
 };
 
 
+
+/***
+ *** Packettype: teamdata
+ *** Server Side:
+ ***    Send all teamdata to the client.
+ *** Client Side:
+ ***    Get all Teamdata
+ ***/
+void do_teamdata (struct pkg_teamdata *td, _net_addr * addr) {
+	int i;
+	
+    if (addr->pl_nr == -1)
+        return;
+	
+	d_printf ("do_teamdata (addr->pl_nr: %d): team:%d col:%d wins:%d\n", addr->pl_nr, td->team_nr, td->col, td->wins);
+	
+	if (addr->pl_nr == bman.p_servnr) { /* packet comes from the server */
+		if (td->team_nr >= 0 && td->team_nr < MAX_TEAMS) {
+			strncpy (teams[td->team_nr].name, td->name, LEN_PLAYERNAME);
+			teams[td->team_nr].col = td->col;
+			teams[td->team_nr].wins = td->wins;
+			bman.updatestatusbar = 1;
+		}
+	}
+	else {
+		if (td->team_nr < 0 || td->team_nr >= MAX_TEAMS) {
+			for (i = 0; i < MAX_TEAMS; i++) 
+				send_teamdata (addr, i);
+		}
+		else {
+			send_teamdata (addr, td->team_nr);
+		}
+	}
+}
+
+
+void send_teamdata (_net_addr * addr, int team_nr) {
+	struct pkg_teamdata td;
+
+	d_printf ("send_teamdata (%s:%s) team:%d\n", addr->host, addr->port, team_nr);
+	
+    td.h.typ = PKG_teamdata;
+    td.h.flags = PKGF_ackreq;
+    td.h.len = HTON16 (sizeof (struct pkg_teamdata));
+
+	td.team_nr = team_nr;
+	if (team_nr >= 0 && team_nr < MAX_PLAYERS) {
+		strncpy (td.name, teams[team_nr].name, LEN_PLAYERNAME);
+		td.wins = teams[team_nr].wins;
+		td.col = teams[team_nr].col;
+	}
+	else {
+		td.name[0] = 0;
+		td.wins = 0;
+		td.col = 0;
+	}
+	send_pkg ((struct pkg *) &td, addr);
+}
+
+
 /***
  *** Packettype: servermode
  ***/
@@ -355,37 +516,38 @@ do_servermode (struct pkg_servermode *s_mod, _net_addr * addr)
     /* if we just have connected the bman.p_nr is still -1, so we handle the
        servermode packet still alittle diffrent */
     if ((!s_mod->lplayer2 && bman.p_nr == -1) || (s_mod->lplayer2 && bman.p2_nr == -1)) {
-        d_printf ("     Server gave us: p_nr: %d, p_servnr: %d\n", s_mod->p_nr, s_mod->p_servnr);
+        d_printf ("     Server gave us: p_nr: %d(old:%d), p_servnr: %d(old:%d)\n", s_mod->p_nr,bman.p_nr, s_mod->p_servnr, bman.p_servnr);
 
         /* set the p_servnr to the playerslot which is the server */
         if (bman.p_servnr != s_mod->p_servnr) {
-            bman.p_servnr = s_mod->p_servnr;
-            memcpy (&players[bman.p_servnr].net, &players[0].net, sizeof (_net_player));
+   	        memcpy (&players[s_mod->p_servnr].net, &players[bman.p_servnr].net, sizeof (_net_player));
+			bzero (&players[bman.p_servnr], sizeof (_net_player));
+			bman.p_servnr = s_mod->p_servnr;
         }
 
         /* set now the new p_nr number */
         if (s_mod->p_nr >= 0 && s_mod->p_nr < MAX_PLAYERS && bman.p_nr == -1 && s_mod->lplayer2 == 0) {
             bman.p_nr = s_mod->p_nr;
+			bman.firewall = 1;
+			players[bman.p_nr].net.flags = NETF_firewall;
             players[bman.p_nr].state &= (0xFF - PSF_net);
             strncpy (players[s_mod->p_nr].name, bman.playername, LEN_PLAYERNAME);
+
+	        /* send playerid with p_nr -1 so we get the whole playerlist
+			 * do the same with the teamdata */
+    	    send_playerid (addr, NULL, NULL, NULL, -1, -1, -1, 0);
+    	    send_teamdata (addr, -1);
         }
         else if (s_mod->p_nr >= 0 && s_mod->p_nr < MAX_PLAYERS && bman.p2_nr == -1 && s_mod->lplayer2 == 1) {
             bman.p2_nr = s_mod->p_nr;
+			players[bman.p2_nr].net.flags = NETF_firewall + NETF_local2;
             players[bman.p2_nr].state &= (0xFF - PSF_net);
             strncpy (players[s_mod->p_nr].name, bman.player2name, LEN_PLAYERNAME);
         }
-
-        /* send playerid with p_nr -1 so we get the whole playerlist */
-        send_playerid (addr, NULL, NULL, NULL, -1, -1, 0);
-    }
-
-    /* the server changed */
-    if (bman.p_servnr != s_mod->p_servnr) {
-		menu_displaymessage ("Server Quit", "The Server Quit the game.");
     }
 
     /* do the normal update */
-    if (GT_MP_PTPS) {
+    if (GT_MP_PTPS && addr->pl_nr == bman.p_servnr) {
         bman.state = s_mod->state;
         bman.gametype = s_mod->gametype;
 		bman.dropitemsondeath = s_mod->dropitemsondeath;
@@ -409,6 +571,29 @@ send_servermode (_net_addr * addr, int pl_nr)
 {
     struct pkg_servermode s_mod;
 
+    switch (bman.state) {
+    case (GS_startup):
+        d_printf ("Send ServerMode : startup\n");
+        break;
+    case (GS_ready):
+        d_printf ("Send ServerMode : ready\n");
+        break;
+    case (GS_running):
+        d_printf ("Send ServerMode : running\n");
+        break;
+    case (GS_quit):
+        d_printf ("Send ServerMode : quit\n");
+        break;
+    case (GS_wait):
+        d_printf ("Send ServerMode : wait\n");
+        break;
+    case (GS_update):
+        d_printf ("Send ServerMode : update\n");
+        break;
+	default:
+		d_printf ("Send ServerMode : mode %d\n", s_mod.state);
+    }
+
     s_mod.h.typ = PKG_servermode;
     s_mod.h.len = HTON16 (sizeof (struct pkg_servermode));
     s_mod.h.flags = PKGF_ackreq;
@@ -431,26 +616,6 @@ send_servermode (_net_addr * addr, int pl_nr)
     s_mod.fieldsize_x = map.size.x;
     s_mod.fieldsize_y = map.size.y;
     strncpy (s_mod.tileset, map.tileset, LEN_TILESETNAME);
-    switch (s_mod.state) {
-    case (GS_startup):
-        d_printf ("Send ServerMode : startup\n");
-        break;
-    case (GS_ready):
-        d_printf ("Send ServerMode : ready\n");
-        break;
-    case (GS_running):
-        d_printf ("Send ServerMode : running\n");
-        break;
-    case (GS_quit):
-        d_printf ("Send ServerMode : quit\n");
-        break;
-    case (GS_wait):
-        d_printf ("Send ServerMode : wait\n");
-        break;
-    case (GS_update):
-        d_printf ("Send ServerMode : wait\n");
-        break;
-    }
     send_pkg ((struct pkg *) &s_mod, addr);
 };
 
@@ -608,7 +773,9 @@ send_playerdata (_net_addr * addr, int p_nr, _player * pl)
     p_dat.dead_by = pl->dead_by;
     p_dat.frame = HTON16 (FTOI16 (pl->frame));
     p_dat.p_nr = p_nr;
-
+	p_dat.ready = pl->ready;
+	p_dat.team_nr = pl->team_nr;
+	
     send_pkg ((struct pkg *) &p_dat, addr);
 };
 
@@ -633,6 +800,7 @@ do_playerdata (struct pkg_playerdata *p_dat, _net_addr * addr)
     if (bman.state == GS_running && bman.p_nr != p_dat->p_nr) {
         pl->points = NTOH16 (p_dat->points);
         pl->dead_by = NTOH16 (p_dat->dead_by);
+		pl->team_nr = p_dat->team_nr;
     }
     else if (bman.state != GS_running || bman.p_nr != p_dat->p_nr) {
         pl->pos.x = I16TOF (NTOH16 (p_dat->pos.x));
@@ -644,6 +812,9 @@ do_playerdata (struct pkg_playerdata *p_dat, _net_addr * addr)
         pl->range = p_dat->range;
         pl->frame = I16TOF (NTOH16 (p_dat->frame));
         pl->state = p_dat->state;
+		pl->team_nr = p_dat->team_nr;
+		pl->ready = p_dat->ready;
+		team_update ();
     }
 
     if (p_dat->dead_by < 0 || p_dat->dead_by >= MAX_PLAYERS) /* set player state no matter what */
@@ -654,6 +825,8 @@ do_playerdata (struct pkg_playerdata *p_dat, _net_addr * addr)
             /* player just died */
             if (p_dat->p_nr != p_dat->dead_by) {
                 players[p_dat->dead_by].points++;
+				if (bman.gametype == GT_team && players[p_dat->dead_by].team_nr >= 0 && players[p_dat->dead_by].team_nr < MAX_TEAMS)
+					teams[players[p_dat->dead_by].team_nr].points++;
                 net_game_send_player (p_dat->dead_by);
             }
         }
@@ -917,7 +1090,7 @@ send_tunneldata (_net_addr * addr, int tunnelnr, int x, int y)
  *** Packettype: quit
  ***/
 void
-send_quit (_net_addr * addr, int pl_nr)
+send_quit (_net_addr * addr, int pl_nr, int new_server)
 {
     struct pkg_quit q_dat;
 
@@ -927,7 +1100,7 @@ send_quit (_net_addr * addr, int pl_nr)
     q_dat.h.flags = 0;
     q_dat.h.len = HTON16 (sizeof (struct pkg_quit));
 	q_dat.pl_nr = pl_nr;
-
+	q_dat.new_server = new_server;
     send_pkg ((struct pkg *) &q_dat, addr);
 };
 
@@ -935,13 +1108,33 @@ send_quit (_net_addr * addr, int pl_nr)
 void
 do_quit (struct pkg_quit *q_dat, _net_addr * addr)
 {
-    d_printf ("do_quit (%s:%s) pl_nr = %d\n", addr->host, addr->port, q_dat->pl_nr);
+    d_printf ("do_quit (%s:%s) pl_nr=%d new_server=%d\n", addr->host, addr->port, q_dat->pl_nr, q_dat->new_server);
 
     if (addr->pl_nr == -1)
         return;
 
-    /* the player who send this quit */
+	bman.updatestatusbar=1;
     player_delete (q_dat->pl_nr);
+
+    /* the player who send this quit */
+	if (q_dat->pl_nr == bman.p_servnr && q_dat->new_server != bman.p_servnr) {
+		d_printf ("do_quit: new server is set to: %d\n", q_dat->new_server);
+		bman.p_servnr = q_dat->new_server;
+		
+		if (GT_MP_PTPM && bman.notifygamemaster) {
+			/* if there are any AI players delete the network flag from them */
+			int i;
+			
+			for (i = 0; i < MAX_PLAYERS; i++)
+				if (PS_IS_aiplayer (players[i].state))
+					players[i].state &= (0xFF - PSF_net);
+			
+			send_ogc_update ();
+		}
+	}
+	else if (q_dat->pl_nr == bman.p_servnr && q_dat->new_server == bman.p_servnr)
+		menu_displaymessage ("Server Quit", "The game closed because you are the only one who is left."
+							 " Or the server could not find any other possible new server.");
 };
 
 
@@ -1546,8 +1739,15 @@ do_pkg (struct pkg *packet, _net_addr * addr)
         return 0;
     }
 	
-    /* get the addr and set the ping timeout value */
+    /* get the addr and set the ping timeout value
+	 * check if the packet is from a player in the game and not from someone else
+	 * this exception is only for PKG_joingame, PKG_error */
     addr->pl_nr = get_player_nr (addr->host, addr->port);
+	if ((addr->pl_nr < 0 || addr->pl_nr >= MAX_PLAYERS) && packet->h.typ > PKG_joingame && PS_IS_netplayer (players[addr->pl_nr].state)) {
+		d_printf ("do_pkg: error addr->pl_nr out of range\n");
+		return 0;
+	}
+
 	if (addr->pl_nr >= 0 && addr->pl_nr < MAX_PLAYERS) {
 		players[addr->pl_nr].net.timestamp = timestamp;
     	players[addr->pl_nr].net.pingreq = players[addr->pl_nr].net.pingack + 5;
@@ -1592,6 +1792,9 @@ do_pkg (struct pkg *packet, _net_addr * addr)
     case (PKG_field):
         do_field ((struct pkg_field *) packet, addr);
         break;
+	case (PKG_contest):
+		do_contest ((struct pkg_contest *) packet, addr);
+		break;
     case (PKG_pingreq):
         do_ping ((struct pkg_ping *) packet, addr);
         break;
@@ -1652,6 +1855,9 @@ do_pkg (struct pkg *packet, _net_addr * addr)
     case (PKG_updateinfo):
         do_updateinfo ((struct pkg_updateinfo *) packet, addr);
         break;
+	case (PKG_teamdata):
+		do_teamdata ((struct pkg_teamdata *) packet, addr);
+		break;
     default:
         send_error (addr, "BomberClone: unknown data packet");
         break;
