@@ -1,4 +1,4 @@
-/* $Id: player.c,v 1.85 2004/06/08 22:35:26 stpohle Exp $
+/* $Id: player.c,v 1.97 2004/12/26 04:19:20 stpohle Exp $
  * player.c - everything what have to do with the player */
 
 #include <SDL.h>
@@ -299,7 +299,6 @@ stepmove_player (int pl_nr)
 	  d;			// distance to move
 	float speed = 0.0f;
 	
-	
     if (p->m == 1) {
         _pos.x = CUTINT(p->pos.x);
         _pos.y = CUTINT(p->pos.y);
@@ -325,8 +324,10 @@ stepmove_player (int pl_nr)
         else
             speed = 1.0f - _pos.y;
 
-        if (speed > (p->stepsleft) || speed == 0)
+        if (speed > (p->stepsleft) || speed == 0.0f)
             speed = p->stepsleft;
+		if (speed > 1.0f)
+			speed = 1.0f;
 
         // check the new field position
         d.x = d.y = 0.0f;
@@ -546,8 +547,11 @@ player_died (_player * player, signed char dead_by)
 
     bman.updatestatusbar = 1;   // force an update
     if (PS_IS_alife (player->state) && dead_by >= 0 && dead_by < MAX_PLAYERS)
-        if (player - players != dead_by)
+        if (player - players != dead_by) {
             players[dead_by].points++;
+			if (bman.gametype == GT_team && players[dead_by].team_nr >= 0 && players[dead_by].team_nr < MAX_TEAMS)
+				teams[players[dead_by].team_nr].points++;
+		}
     player->frame = 0;
     player->state &= (0xFF - PSF_alife);
     player->dead_by = dead_by;
@@ -824,6 +828,8 @@ player_clear_ilness (_player * p, int type)
 void
 player_set_gfx (_player * p, signed char gfx_nr)
 {
+	d_printf ("player_set_gfx: name:%15s from gfx %d to gfx %d.\n", p->name, p->gfx_nr, gfx_nr);
+	
     p->gfx_nr = gfx_nr;
     if (p->gfx_nr < 0 || p->gfx_nr >= MAX_PLAYERS)
         p->gfx_nr = -1;
@@ -835,6 +841,7 @@ player_set_gfx (_player * p, signed char gfx_nr)
         p->gfx = &gfx.players[gfx_nr];
         p->state |= PSF_playing;
     }
+	if (GT_MP) net_send_playerid (p-players);
 };
 
 
@@ -965,9 +972,6 @@ void player_check (int pl_nr) {
  */
 void
 player_delete (int pl_nr) {
-    char host[LEN_SERVERNAME];
-    char port[LEN_PORT];
-
     d_printf ("player_delete (%d)\n", pl_nr);
 	bman.updatestatusbar = 1;   // force an update
 
@@ -978,10 +982,7 @@ player_delete (int pl_nr) {
         bman.state = GS_startup;
     }
     else {
-        strncpy (host, players[pl_nr].net.addr.host, LEN_SERVERNAME);
-        strncpy (port, players[pl_nr].net.addr.port, LEN_PORT);
-
-		player_set_gfx (&players[pl_nr], -1);
+    	player_set_gfx (&players[pl_nr], -1);
         players[pl_nr].state &= (0xFF - (PSF_used + PSF_net + PSF_alife + PSF_ai)); /* delete 
 												player flags */
         players[pl_nr].net.net_istep = 0; // needed for disconnect during the update
@@ -992,9 +993,6 @@ player_delete (int pl_nr) {
 		net_game_send_delplayer (pl_nr);
 	}
 
-    if (GT_MP_PTPS && pl_nr == bman.p_servnr) /* server quit */
-        bman.state = GS_startup;
-
     if (GT_MP_PTPM && bman.notifygamemaster)
         send_ogc_update ();
 	
@@ -1036,11 +1034,7 @@ void player2_join () {
 		}
 	}
 	else if (GT_MP_PTPS) {	/* multiplayer client, send only the request */
-		int flags = NETF_local2;
-		
-		if (bman.firewall)
-			flags |= NETF_firewall;
-		send_joingame (&players[bman.p_servnr].net.addr, bman.player2name, flags);
+		send_joingame (&players[bman.p_servnr].net.addr, bman.player2name, 1);
 	}
 	else { 		/* single mode */
 		for (i = 0; PS_IS_used (players[i].state) && i < MAX_PLAYERS; i++);
@@ -1062,5 +1056,44 @@ void player2_add (int pl_nr) {
 	players[pl_nr].gfx = NULL;
 	players[pl_nr].points = 0;
 	players[pl_nr].wins = 0;
+	players[pl_nr].team_nr = -1;
 	d_printf ("player2_add: Local Player Added with pl_nr: %d\n", pl_nr);
 };
+
+
+/*
+ * set the teams[] with the current player data
+ */
+void team_update () {
+	int cnt[MAX_TEAMS] = { 0,0,0,0 };
+	int pl;
+	
+	for (pl = 0; pl < MAX_PLAYERS; pl++) {
+		if (PS_IS_used (players[pl].state) && (players[pl].team_nr < 0 || players[pl].team_nr >= MAX_TEAMS))
+			players[pl].team_nr = 0;
+		
+		if (PS_IS_used (players[pl].state)) {
+			teams[players[pl].team_nr].players[cnt[players[pl].team_nr]] = &players[pl];
+			cnt[players[pl].team_nr]++;
+		}
+	}
+	for (pl = 0; pl < MAX_TEAMS; pl++) for (;cnt[pl] < MAX_PLAYERS; cnt[pl]++)
+		teams[pl].players[cnt[pl]] = NULL;
+	
+	// d_teamdetail ("Teaminformation");
+}
+
+
+/*
+ * if the team number is -1 then join this player into the losing team
+ */
+void team_choose (_player *pl) {
+	
+	if (pl->team_nr >= 0 && pl->team_nr < MAX_TEAMS)
+		return;
+	
+	d_printf ("team_choose need to rewrite this part\n");
+	if (PS_IS_aiplayer (pl->state))
+		pl->team_nr = 1;
+	else pl->team_nr = 0;
+};
