@@ -1,4 +1,4 @@
-/* $Id: single.c,v 1.67 2004/06/15 15:04:37 stpohle Exp $ */
+/* $Id: single.c,v 1.82 2004/12/26 04:19:20 stpohle Exp $ */
 /* single player */
 
 #include "basic.h"
@@ -37,6 +37,51 @@ single_game_new ()
 };
 
 
+
+/*
+ * the ai player will choose diffrent gfx for every
+ * team another gfx, but not one which is selected by
+ * human player.
+ */
+void ai_team_choosegfx () {
+	struct _team_tmpdata_ {
+		int ai_gfx;
+	} teamdat[MAX_TEAMS];
+	
+	int tm_nr;
+	int pl_nr;
+	int i, used;
+	_player *pl;
+	
+	/* find a ai player gfx for every team */
+	for (i = 0, tm_nr = 0; (i < gfx.player_gfx_count && tm_nr < MAX_TEAMS); i++) {
+		used = 0;
+		for (pl_nr = 0; pl_nr < MAX_PLAYERS; pl_nr++) {
+			if ((!PS_IS_aiplayer(players[pl_nr].state)) 
+				&& (PS_IS_used (players[pl_nr].state)) 
+				&& i == players[pl_nr].gfx_nr)
+					used = 1;
+		}
+		if (!used && tm_nr < MAX_TEAMS) {
+			teamdat[tm_nr].ai_gfx = i;
+			tm_nr++;
+		}
+	}
+	
+	/*
+	 * give all ai players in the teams the right gfx
+	 */
+	for (tm_nr = 0; tm_nr < MAX_TEAMS; tm_nr++) for (pl_nr = 0; pl_nr < MAX_PLAYERS; pl_nr++) {
+		pl = teams[tm_nr].players[pl_nr];
+		if (pl) {
+			if (PS_IS_used(pl->state) && PS_IS_aiplayer(pl->state))
+				player_set_gfx (pl, teamdat[tm_nr].ai_gfx);
+		}
+	}
+};
+
+
+
 inline int
 ai_checkfield (int x, int y)
 {
@@ -51,42 +96,24 @@ ai_checkfield (int x, int y)
  * this function is even needed for the start of 
  * the game to place the players on a good position */
 int
-ai_easyrunaway (_point p)
+ai_easyrunaway (_point p, int range)
 {
     int i,
+	  dist = 0,
       done = 0,
         dir = 0;
-    _point pos[4],
-      m[4];
+    _point pos[4];
 
     for (i = 0; i < 4; i++) {
         pos[i] = p;
-        switch (i) {
-        case (left):
-            m[i].x = -1;
-            m[i].y = 0;
-            break;
-        case (right):
-            m[i].x = 1;
-            m[i].y = 0;
-            break;
-        case (up):
-            m[i].x = 0;
-            m[i].y = -1;
-            break;
-        default:
-            m[i].x = 0;
-            m[i].y = 1;
-            break;
-        }
-        pos[i].x += m[i].x;
-        pos[i].y += m[i].y;
+        pos[i].x += dir_change[i].x;
+        pos[i].y += dir_change[i].y;
     }
 
     /* test the possible ways */
     while (!done) {
         done = 1;
-
+		dist++;
         for (i = 0; i < 4; i++) {
             /* check if we are still in the game field */
             if (pos[i].x <= 0 || pos[i].y <= 0 || pos[i].x >= map.size.x - 1
@@ -110,8 +137,9 @@ ai_easyrunaway (_point p)
                                 || ai_checkfield (pos[i].x + 1, pos[i].y)))
                             dir |= (1 << i);
                     }
-                    pos[i].x += m[i].x;
-                    pos[i].y += m[i].y;
+                    pos[i].x += dir_change[i].x;
+                    pos[i].y += dir_change[i].y;
+					if (dist > range) dir |= (1 << i);
                 }
             }
         }
@@ -124,7 +152,7 @@ ai_easyrunaway (_point p)
 /* give the run away direction 
   the return value:				*/
 _airunaway
-ai_runawayfrom (_point p, int nearbomb, signed char norecursive)
+ai_runawayfrom (_point p, int nearbomb, int range, signed char norecursive)
 {
     int i,
       done = 0,
@@ -133,10 +161,10 @@ ai_runawayfrom (_point p, int nearbomb, signed char norecursive)
         _i,
         bdirpoints = 10,
         j,
-        c;
+        c,
+		dist = 0;
     _airunaway res;
     _point pos[4],
-      m[4],
       tpos;
 
     res.dir = 0;
@@ -144,26 +172,8 @@ ai_runawayfrom (_point p, int nearbomb, signed char norecursive)
 
     for (i = 0; i < 4; i++) {
         pos[i] = p;
-        switch (i) {
-        case (left):
-            m[i].x = -1;
-            m[i].y = 0;
-            break;
-        case (right):
-            m[i].x = 1;
-            m[i].y = 0;
-            break;
-        case (up):
-            m[i].x = 0;
-            m[i].y = -1;
-            break;
-        default:
-            m[i].x = 0;
-            m[i].y = 1;
-            break;
-        }
-        pos[i].x += m[i].x;
-        pos[i].y += m[i].y;
+        pos[i].x += dir_change[i].x;
+        pos[i].y += dir_change[i].y;
     }
 
     /* test if we just have to move to the side */
@@ -178,7 +188,7 @@ ai_runawayfrom (_point p, int nearbomb, signed char norecursive)
     /* test the possible ways */
     while (!done) {
         done = 1;
-
+		dist++;
         for (i = 0; i < 4; i++) {
             /* check if we are still in the game field */
             if (pos[i].x <= 0 || pos[i].y <= 0 || pos[i].x >= map.size.x - 1
@@ -194,13 +204,13 @@ ai_runawayfrom (_point p, int nearbomb, signed char norecursive)
                         if (((i == left || i == right) && (j == up || j == down)) ||
                             ((j == left || j == right) && (i == up || i == down))) {
                             c = 10;
-                            tpos.x = pos[i].x + m[j].x;
-                            tpos.y = pos[i].y + m[j].y;
+                            tpos.x = pos[i].x + dir_change[j].x;
+                            tpos.y = pos[i].y + dir_change[j].y;
                             if (ai_checkfield (tpos.x, tpos.y)) {
                                 nbomb = ai_findnearbombs (tpos);
                                 c = s_countbits (nbomb, 5);
                                 if (!norecursive) {
-                                    tdir = ai_runawayfrom (tpos, nbomb, 1).dir;
+                                    tdir = ai_runawayfrom (tpos, nbomb, range, 1).dir;
                                     _i = ai_invertdir (i);
                                     if (tdir != (1 << _i)) { // usefull direction
                                         res.dir |= (1 << i); // add this one
@@ -224,8 +234,12 @@ ai_runawayfrom (_point p, int nearbomb, signed char norecursive)
                                 }
                             }
                         }
-                    pos[i].x += m[i].x;
-                    pos[i].y += m[i].y;
+                    pos[i].x += dir_change[i].x;
+                    pos[i].y += dir_change[i].y;
+					if (dist > range && res.bestdir == -1) {
+						res.dir |= (1 << i);
+						res.bestdir = i;
+					}
                 }
             }
         }
@@ -242,39 +256,19 @@ ai_bombpoints (_point pos, int range)
     int points = 0,
         d,
         r;
-    _point p,
-      m;
+    _point p;
 
     if (map.field[pos.x][pos.y].type != FT_block && map.field[pos.x][pos.y].type != FT_stone
         && ai_checkfield (pos.x, pos.y)) {
         for (d = 0; d < 4; d++) {
-            switch (d) {
-            case (left):
-                m.x = -1;
-                m.y = 0;
-                break;
-            case (right):
-                m.x = 1;
-                m.y = 0;
-                break;
-            case (up):
-                m.x = 0;
-                m.y = -1;
-                break;
-            default:
-                m.x = 0;
-                m.y = 1;
-                break;
-            }
-
             p = pos;
 
             for (r = 0;
                  (r < range
                   && (map.field[p.x][p.y].type == FT_nothing
                       || map.field[p.x][p.y].type == FT_tunnel)); r++) {
-                p.x += m.x;
-                p.y += m.y;
+                p.x += dir_change[d].x;
+                p.y += dir_change[d].y;
             }
 
             if (map.field[p.x][p.y].type != FT_nothing && map.field[p.x][p.y].type != FT_tunnel
@@ -287,7 +281,7 @@ ai_bombpoints (_point pos, int range)
         }
     }
 
-    if (ai_easyrunaway (pos) == 0 || ai_findnearbombs (pos) != 0)
+    if (ai_easyrunaway (pos, range) == 0 || ai_findnearbombs (pos) != 0)
         points = 0;
 
     return points;
@@ -304,33 +298,13 @@ ai_findbestbombdir (_point pos, int dir, int range)
         j,
         maxpoints = 0,
         bestd;
-    _point m[4],
-      p[4];
+    _point p[4];
 
     points[4] = ai_bombpoints (pos, range);
 
     /* fill in the positions */
     for (d = 0; d < 4; d++) {
         p[d] = pos;
-
-        switch (d) {
-        case (left):
-            m[d].x = -1;
-            m[d].y = 0;
-            break;
-        case (right):
-            m[d].x = 1;
-            m[d].y = 0;
-            break;
-        case (up):
-            m[d].x = 0;
-            m[d].y = -1;
-            break;
-        default:
-            m[d].x = 0;
-            m[d].y = 1;
-            break;
-        }
     }
 
     while (!done) {
@@ -339,8 +313,8 @@ ai_findbestbombdir (_point pos, int dir, int range)
         for (d = 0; d < 5; d++) {
 
             if (d < 4) {
-                p[d].x += m[d].x;
-                p[d].y += m[d].y;
+                p[d].x += dir_change[d].x;
+                p[d].y += dir_change[d].y;
                 if (p[d].x > 0 && p[d].y > 0 && p[d].x < map.size.x - 1 && p[d].y < map.size.y - 1) {
                     if (ai_checkfield (p[d].x, p[d].y)) {
                         /* we are opn a empty field go on with the test */
@@ -383,37 +357,13 @@ ai_findnearbombs (_point pos)
     int d,
       res = 0,                  // result if there is a bomb
         done = 0;
-    _point m[4];                // direction addition
     _point dist[4];             // to check every direction (on three ways)
 
-    for (d = 0; d < 4; d++)
-        switch (d) {
-        case (left):
-            m[d].x = -1;
-            m[d].y = 0;
-            dist[d].x = pos.x - 1;
-            dist[d].y = pos.y;
-            break;
-        case (right):
-            m[d].x = 1;
-            m[d].y = 0;
-            dist[d].x = pos.x + 1;
-            dist[d].y = pos.y;
-            break;
-        case (up):
-            m[d].x = 0;
-            m[d].y = -1;
-            dist[d].x = pos.x;
-            dist[d].y = pos.y - 1;
-            break;
-        case (down):
-            m[d].x = 0;
-            m[d].y = 1;
-            dist[d].x = pos.x;
-            dist[d].y = pos.y + 1;
-            break;
-        }
-
+    for (d = 0; d < 4; d++) {
+        dist[d].x = pos.x + dir_change[d].x;
+        dist[d].y = pos.y + dir_change[d].y;
+	}
+	
     while (!done) {
         done = 1;
         /* check every direction again */
@@ -427,8 +377,8 @@ ai_findnearbombs (_point pos)
                 if (map.field[dist[d].x][dist[d].y].type != FT_nothing)
                     dist[d].x = dist[d].y = -1; // don't check no more.
                 else if (dist[d].x != -1 && dist[d].y != -1) {
-                    dist[d].x += m[d].x;
-                    dist[d].y += m[d].y;
+                    dist[d].x += dir_change[d].x;
+                    dist[d].y += dir_change[d].y;
                     done = 0;
                 }
             }
@@ -523,59 +473,59 @@ inline int
 ai_checknewpos (_point pos, int d)
 {
     _point m;
-    switch (d) {
-    case (left):
-        m.x = pos.x - 1;
-        m.y = pos.y;
-        break;
-    case (right):
-        m.x = pos.x + 1;
-        m.y = pos.y;
-        break;
-    case (up):
-        m.x = pos.x;
-        m.y = pos.y - 1;
-        break;
-    default:
-        m.x = pos.x;
-        m.y = pos.y + 1;
-        break;
-    }
+	
+	m.x = pos.x + dir_change[d].x;
+	m.y = pos.y + dir_change[d].y;
 
     return (ai_findnearbombs (m) == 0);
 };
 
 
 /* create a giving number of ai players */
-void
+int
 single_create_ai (int num_players)
 {
     int p,
       count,
+	  try,
       gfx_sel,
       i = 0;
-    _player *pl;
+    _player *pl = NULL;
 
     for (count = 0; count < num_players; count++) {
         /* find free players */
-        for (pl = NULL, p = 0; (pl == NULL && p < MAX_PLAYERS); p++)
-            if (!(PS_IS_used (players[p].state))) {
+        for (pl = NULL, p = 0; (pl == NULL && p < MAX_PLAYERS);p++)
+            if (p < MAX_PLAYERS && !(PS_IS_used (players[p].state))) {
                 pl = &players[p];
                 sprintf (pl->name, "AI %d", p + 1);
                 pl->state |= PSF_used + PSF_alife + PSF_playing + PSF_ai;
                 pl->net.flags = NETF_firewall;
                 sprintf (pl->net.addr.host, "localhost");
                 sprintf (pl->net.addr.port, "11000");
+				try = 0;
                 do {
-                    gfx_sel = s_random (MAX_PLAYERS);
+                    gfx_sel = s_random (gfx.player_gfx_count);
                     MW_IS_GFX_SELECT (gfx_sel, i);
-                } while (i != -1);
+					try++;
+                } while (try < 100 && i != -1);
+				pl->wins = 0;
+				pl->points = 0;
+				pl->team_nr = -1;
                 player_set_gfx (pl, gfx_sel);
-            }
+				team_choose (pl);
+			}
 
-        if (pl == NULL)
-            return;
-    }
+        if (pl == NULL) {
+			d_printf ("single_create_ai: No free Player found.\n"); 
+            return -1;
+		}
+		else
+			d_printf ("single_create_ai: Player %d Created  Name:%s\n", pl - players, pl->name); 
+	}
+	
+	if (pl != NULL)
+		return pl - players;
+	return -1;	
 };
 
 
@@ -593,19 +543,30 @@ single_playergame (int second_player, int ai_players)
 	bman.p_nr = -1;
 	bman.p2_nr = -1;
 	
-    /* delete player from the game */
+    /* delete player and teams from the game */
     for (p = 0; p < MAX_PLAYERS; p++) {
         players[p].points = 0;
         players[p].wins = 0;
         players[p].state = 0;
+		players[p].team_nr = -1;
         players[p].gfx_nr = -1;
         players[p].gfx = NULL;
 		memset (&players[p].net, 0x0, sizeof (_net_player));
     }
 
+	for (p = 0; p < MAX_TEAMS; p++) {
+		teams[p].wins = 0;
+		teams[p].points = 0;
+		
+		for (done = 0; done < MAX_PLAYERS; done++)
+			teams[p].players[done] = NULL;
+	}
+	
+	done = 0;
     for (bman.p_nr = -1, p = 0; (bman.p_nr == -1 && p < MAX_PLAYERS); p++)
         if (!(PS_IS_used (players[p].state)))
             bman.p_nr = p;
+	players[bman.p_nr].team_nr = 0;
 
     if (bman.p_nr >= MAX_PLAYERS) {
         printf ("ERROR in function (single_game_new): couldn't find any free player\n");
@@ -613,23 +574,32 @@ single_playergame (int second_player, int ai_players)
     }
 
 	strncpy (players[bman.p_nr].name, bman.playername, LEN_PLAYERNAME);
-	gfx_load_players (32, 32);
 	do {
-		playermenu_selgfx (bman.p_nr);
-	} while (players[bman.p_nr].gfx_nr == -1);
-	
+		done = playermenu_selgfx (bman.p_nr);
+	} while (players[bman.p_nr].gfx_nr == -1 && done != -1);
+		
     players[bman.p_nr].state = PSF_used + PSF_alife + PSF_playing;
     strncpy (players[bman.p_nr].name, bman.playername, LEN_PLAYERNAME);
 
-	if (second_player) {
+	if (done != -1 && second_player) {
 		player2_join ();
 		do {
-			playermenu_selgfx (bman.p2_nr);
-		} while (players[bman.p2_nr].gfx_nr == -1);
+			done = playermenu_selgfx (bman.p2_nr);
+		} while (players[bman.p2_nr].gfx_nr == -1 && done != -1);
+		players[bman.p2_nr].team_nr = 0;
 	}
-	gfx_free_players ();
 
+	if (done == -1)
+		return;
+	
     single_create_ai (ai_players);
+	
+	if (bman.gametype == GT_team) {
+		playermenu ();
+		team_update ();
+		ai_team_choosegfx ();
+	}
+	
     bman.state = GS_ready;
 
     while (!done && bman.state != GS_quit && bman.state != GS_startup) {
@@ -676,7 +646,7 @@ single_loop ()
                     if (nearbomb == 0) { // no bombs found
                         bestbdir = ai_findbestbombdir (plpos, pl->d, pl->range);
                         if (bestbdir & DIRM_under) {
-                            if (ai_easyrunaway (plpos) != 0)
+                            if (ai_easyrunaway (plpos, pl->range) != 0)
                                 player_drop_bomb (p);
                         }
                         else if (bestbdir == 0) {
@@ -692,7 +662,7 @@ single_loop ()
                     }
                     else {
                         // bombs in the near found
-                        rawdir = ai_runawayfrom (plpos, nearbomb, 0);
+                        rawdir = ai_runawayfrom (plpos, nearbomb, pl->range, 0);
                         if (rawdir.dir != 0 && rawdir.bestdir == -1) {
                             pl->d = ai_choosedir (rawdir.dir, nearbomb, pl->d); // we have to make a choice.. do it
                             pl->m = 1;
@@ -700,7 +670,13 @@ single_loop ()
                         else if (rawdir.bestdir != -1) {
                             pl->d = rawdir.bestdir;
                             pl->m = 1;
-                        }
+                        } 
+						else if (rawdir.bestdir == -1 && rawdir.dir == 0) {
+							/* no good ways found, just run in the opposite direction of the bomb */
+							if (map.field[(int) pl->pos.x][(int) pl->pos.y].type == FT_nothing ||
+								map.field[(int) pl->pos.x][(int) pl->pos.y].type >= FT_tunnel)
+									pl->m = 1;
+						}
                     }
 
                     if (pl->m == 0 && map.field[(int) pl->pos.x][(int) pl->pos.y].type == FT_tunnel)
@@ -721,24 +697,18 @@ void
 single_menu ()
 {
     int i,
-		p, 
+		p,
 		eventstate = 0,
         done = 0,
-		second_playerold = 0,
 		second_player = 0;
     _charlist nrplayerlist[MAX_PLAYERS + 1];
     _charlist *selnrplayer = &nrplayerlist[bman.ai_players];
 	_menu *menu;
-	_menuitem *pl2name = NULL;
 	_menuitem *aiplayer = NULL;
 	SDL_Event event;
 
     /* fill in the nrplayerlist */
-    if (debug)
-        p = 0;
-    else
-        p = 1;
-    for (i = 0; p < MAX_PLAYERS + 1; i++) {
+    for (p = 0, i = 0; p < MAX_PLAYERS + 1; i++) {
         sprintf (nrplayerlist[i].text, "%d", p);
         if (p < MAX_PLAYERS - 1)
             nrplayerlist[i].next = &nrplayerlist[i + 1];
@@ -752,12 +722,12 @@ single_menu ()
 	menu_create_text (menu, "numpl", 20, 50, 12, 2, COLOR_yellow, "Number of\nAI Players");
 	aiplayer = menu_create_list (menu, "AI Players", 40, 90, 50, 100, nrplayerlist, &selnrplayer, 3);
 	
-	menu_create_entry (menu,"Player 1:",150, 50, 230, &bman.playername, LEN_PLAYERNAME, MENU_entrytext, 4);
+	menu_create_button (menu,"Change Playernames" ,160, 50, 210, 4);
 
 	menu_create_bool (menu, "Use Second Player", 160, 90, 210, &second_player, 5);
 
-	menu_create_button (menu, "Map Options", 180, 160, 150, 7);
-	
+	menu_create_button (menu, "Game Options", 180, 130, 150, 6);
+	menu_create_button (menu, "Map Options", 180, 170, 150, 7);
 	menu_create_button (menu, "Main Menu", 30, 220, 150, 1);
 	menu_create_button (menu, "Start", 220, 220, 150, 2);
 	
@@ -766,31 +736,27 @@ single_menu ()
 	menu_draw (menu);
 	
 	do {
-		/* create or destroy second playername menuentry */
-		if (second_player != second_playerold) {
-			if (pl2name != NULL && !second_player) {
-				menu_del_menuitem (pl2name);
-				pl2name = NULL;
-			}
-			if (pl2name == NULL && second_player)
-				pl2name = menu_create_entry (menu,"Player 2:",150,120, 230, &bman.player2name, LEN_PLAYERNAME, MENU_entrytext, 6);
-			
-			second_playerold = second_player;
-			aiplayer->changed = 1;
-			menu_draw (menu);
-		}
-
 		gfx_blitdraw ();
 		eventstate = SDL_PollEvent (&event);
-		
+
 		done = menu_event_loop (menu, &event, eventstate);
+
+		if (done > 0 && menu->focus->id == 6) {
+			done = 0;
+			mapgamesetting ();
+		}
 		
-		if (done && menu->focus->id == 7) {
+		if (done > 0 && menu->focus->id == 7) {
 			done = 0;
 			mapmenu ();
 		}
 		
-		if (done && menu->focus->id == 3)
+		if (done > 0 && menu->focus->id == 4) {
+			done = 0;
+			playernamemenu ();
+		}
+		
+		if (done > 0 && menu->focus->id == 3)
 			done = 0;
 		
 		s_calctimesync ();
